# -*- coding: utf-8 -*-
"""
Created on Mon Nov 21 11:17:54 2016
this script contains functions that are needed for tracking the animals in the videos
indicated in the level1 table. some functions also calculate some further parameters
based on the tracking, such as the distance to the image edge or the distance
to the nearest wall.

@author: haenzi
"""

import cv2
import numpy as np
#from matplotlib import pyplot as plt
import os
from os import path
import pandas as pd
from rdp import rdp


def distanceToImageEdge(coordinate, imgShape, tankSize):
    tankSizemm = tankSize*10
    # watch out: image shape is (maxHeight, maxWidth) i.e. (y, x)!
    xFactor = imgShape[1]/tankSizemm # this is in pixels/mm
    yFactor = imgShape[0]/tankSizemm
    minDist = min(coordinate[0]/xFactor, coordinate[1]/yFactor, 
                  (imgShape[1]-coordinate[0])/xFactor, (imgShape[0]-coordinate[1])/yFactor)
    return minDist
    
def distanceToImageEdgePixels(coordinate, imgShape):
    minDist = min(coordinate[0], coordinate[1], imgShape[1] - coordinate[0], imgShape[0] - coordinate[1])
    return minDist
    
def distCoveredMM(coordinate0, coordinate1, imgShape, tankSize):
    tankSizemm = tankSize*10
    xFactor = imgShape[0]/tankSizemm # this is in pixels/mm
    yFactor = imgShape[1]/tankSizemm
    x0 = coordinate0[0]/xFactor
    y0 = coordinate0[1]/yFactor
    x1 = coordinate1[0]/xFactor
    y1 = coordinate1[1]/yFactor
    dist = np.sqrt((x1-x0)**2 + (y1-y0)**2)
    return dist
    
def distCoveredPixel(coordinate0, coordinate1):
    x0 = coordinate0[0]
    y0 = coordinate0[1]
    x1 = coordinate1[0]
    y1 = coordinate1[1]
    dist = np.sqrt((x1-x0)**2 + (y1-y0)**2)
    return dist
    
def pivotAngle3points(coordinate0, coordinate1, coordinate2):
    #define 3 sides of triangle from 3 points
    a = np.sqrt((coordinate2[1] - coordinate1[1])**2 + (coordinate2[0] - coordinate1[0])**2);
    b = np.sqrt((coordinate2[1] - coordinate0[1])**2 + (coordinate2[0] - coordinate0[0])**2);
    c = np.sqrt((coordinate1[1] - coordinate0[1])**2 + (coordinate1[0] - coordinate0[0])**2);
    if (a == 0) or (c == 0):
        turningAngle = 0 #i.e. animal did not move
    else:
        beta = np.arccos((a**2 + c**2 - b**2)/(2*a*c))
        beta = np.degrees(beta)
        turningAngle = 180 - beta
    return turningAngle
    
def externalAngle(coordinate0, coordinate1):
    x = coordinate1[0] - coordinate0[0]
    y = coordinate1[1] - coordinate0[1]
    angleRad = np.arctan2(y,x)
    angleDeg = np.degrees(angleRad)
    return angleDeg


def tadpolePosAngle(img, thresh): #, previousCoordinate):
    """
    function that takes a black-and-white image currently in the workspace, 
    thresholds it (simple binary threshold; threshold value as input), 
    calculates the contours, takes the second-largest contour and outputs
    its position and angle
    """
    
    # define output dict
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    
    # threshold
    ret,th1 = cv2.threshold(img,thresh,255,cv2.THRESH_BINARY)
    
    # find contours
    im2, contours, hierarchy = cv2.findContours(th1,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
    
    # make a list of the contour areas
    cArea = [cv2.contourArea(c) for c in contours]
    
    """
    
    # find the contours larger than 4 and smaller than 1/100th of the tank
    matches = [i for i, x in enumerate(cArea) if x > 3 and x < img.size/2]
    
    # take the largest of these to be the tadpole
    cArea = np.array(cArea)
    tadpoleIndexMatches = cArea[matches].argmax()
    tadpoleIndex = matches[tadpoleIndexMatches]
    tadpoleContour = contours[tadpoleIndex]
    """
    
    # repeat until contour found that is not further from previous coordinate than maxDist
    """
    contourChoice = 2
    repeat = True
    maxDist = img.shape[0]/4 # a quarter of the image
    while repeat:
    """
    contourChoice = 2 #tadpole is (hopefully) the second-largest contour
    try:
        tadpoleIndex = cArea.index(sorted(cArea)[-contourChoice])
        tadpoleContour = contours[tadpoleIndex]
        M = cv2.moments(tadpoleContour)
        d['tadpoleXcontour'] = int(M['m10']/M['m00'])
        d['tadpoleYcontour'] = int(M['m01']/M['m00'])
        
        #fit an ellipse to the tadpole
        ellipse = cv2.fitEllipse(tadpoleContour)
        d['tadpoleXellipse'] = ellipse[0][0]
        d['tadpoleYellipse'] = ellipse[0][1]
        d['ellipseAngle'] = ellipse[-1]
    except: # if it does not work (most likely because there were too few points) try again with higher thresh
        ret,th1 = cv2.threshold(img,thresh+40,255,cv2.THRESH_BINARY)
        im2, contours, hierarchy = cv2.findContours(th1,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
        cArea = [cv2.contourArea(c) for c in contours]
        tadpoleIndex = cArea.index(sorted(cArea)[-contourChoice])
        tadpoleContour = contours[tadpoleIndex]
        M = cv2.moments(tadpoleContour)
        d['tadpoleXcontour'] = int(M['m10']/M['m00'])
        d['tadpoleYcontour'] = int(M['m01']/M['m00'])
        # fit ellipse
        ellipse = cv2.fitEllipse(tadpoleContour)
        d['tadpoleXellipse'] = ellipse[0][0]
        d['tadpoleYellipse'] = ellipse[0][1]
        d['ellipseAngle'] = ellipse[-1]
            
        """
        if np.isnan(previousCoordinate[0]):
            repeat = False
        elif ((d['tadpoleXcontour'] - previousCoordinate[0]) < maxDist) or ((d['tadpoleYcontour'] - previousCoordinate[1]) < maxDist):
            repeat = False
        else:
            contourChoice += 1
            """
            
    
    # check whether ellipse and contour are in the same place... but better to 
    # do this later, when I can also say at which frame this was the case    
    #if (abs(d['tadpoleXellipse'] - d['tadpoleXcontour']) > 5) or (abs(d['tadpoleYellipse'] - d['tadpoleYcontour']) > 5):
    #    print('ellipse and contour coordinates are more than 5 pixels apart!')
    
    return d, ellipse
    
def tadpolePosAngleGaussian(img, bg, kernel, param1, param2, closingTrue): #, previousCoordinate):
    """
    function that takes a black-and-white image currently in the workspace, 
    thresholds it using a Gaussian threshold, closes it using the kernel, 
    calculates the contours, takes the second-largest contour and outputs
    its position and angle
    """
    
    # define output dict
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    
    # option 2: gaussian plus closure
    th2 = cv2.adaptiveThreshold(img,255,cv2.ADAPTIVE_THRESH_GAUSSIAN_C,\
            cv2.THRESH_BINARY,param1,param2)
    if closingTrue:
        th2 = cv2.morphologyEx(th2, cv2.MORPH_CLOSE, kernel)
    
    # subtract WB background
    subtracted0 = bg - th2
    subtracted1 = 255 - subtracted0
    
    # add white padding: 2 pixels on each side
    padded = 255*np.ones((img.shape[0]+4, img.shape[1]+4), dtype='uint8')
    padded[2:-2, 2:-2] = subtracted1
    
    # find contours
    im2, contours, hierarchy = cv2.findContours(padded,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
    
    # make a list of the contour areas
    cArea = [cv2.contourArea(c) for c in contours]
    
    contourChoice = 2 #tadpole is (hopefully) the second-largest contour
    try:
        tadpoleIndex = cArea.index(sorted(cArea)[-contourChoice])
        tadpoleContour = contours[tadpoleIndex]
        M = cv2.moments(tadpoleContour)
        d['tadpoleXcontour'] = int(M['m10']/M['m00'])
        d['tadpoleYcontour'] = int(M['m01']/M['m00'])
        
        #fit an ellipse to the tadpole
        ellipse = cv2.fitEllipse(tadpoleContour)
        d['tadpoleXellipse'] = ellipse[0][0]
        d['tadpoleYellipse'] = ellipse[0][1]
        d['ellipseAngle'] = ellipse[-1]
    except:
        d['tadpoleXcontour'] = np.nan
        d['tadpoleYcontour'] = np.nan
        # fit ellipse
        d['tadpoleXellipse'] = np.nan
        d['tadpoleYellipse'] = np.nan
        d['ellipseAngle'] = np.nan
        ellipse = np.nan
            
    return d, ellipse



def trackTadpoleWarp(dataPath, imageCheckFolder, warpMatrix, maxWidth, maxHeight, thresh):
    
    imList = os.listdir(dataPath)
    if len(imList) > 9999:
        imList = sorted(imList, key=lambda im: int(im[(im.rfind('-') + 1):im.rfind('.')]))
    else:
        imList.sort()
    
    # initialise variable to hold output
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    df = pd.DataFrame(d)
    
    os.chdir(imageCheckFolder)
    
    for index,im in enumerate(imList[0:(len(imList))]):
        if np.mod(index, 1000) == 0:
            print('currently at frame ', index)
        
        # load
        imPath = path.join(dataPath, im)
        img = cv2.imread(imPath,0)
        
        # warp
        warp = cv2.warpPerspective(img, warpMatrix, (maxWidth, maxHeight))
        
        # track
        try:
            currentPos, currentEllipse = tadpolePosAngle(warp, thresh)
            df.loc[df.shape[0]] = currentPos
            
            # save image with coordinate to check in checkImagesPath
            cv2.circle(warp, (int(currentPos['tadpoleXcontour']), int(currentPos['tadpoleYcontour'])), 2, 230, 2)
            cv2.ellipse(warp, currentEllipse, 20, 2)
        except:
            df.loc[df.shape[0]] = np.nan
            print('added nan value at index ', index)
        
        #imPath = path.join(checkImagesPath, 'im'+str(index)+'.tif')
        cv2.imwrite('im'+str(index)+'.tif', warp)
        
        del(img, warp)
        
        
    return df
    
    
def findBGmanual(dataPath):
    
    # find image with animal in centre.. ask user whether the shown image is ok
    imList = os.listdir(dataPath)
    imList.sort()
    ok = False
    myIndex = 0
    while not ok:
        imName = imList[myIndex]
        imPath = path.join(dataPath, imName)
        img = cv2.imread(imPath,0)
        showImage(img, 'is ok?')
        isOK = input('is this image ok? y/n ')
        if isOK == 'y':
            ok = True
        elif isOK == 'n':
            myIndex += 100
        else:
            print('got some strange input, try again')
            
    # ask user to click to the top left and bottom right of the animal
    anCorners, anBox = findAnimalPos(img)
    
    # set pixels in this rectangle to the mean of the surrounding pixels
    x1 = anCorners[0][0]
    x2 = anCorners[1][0]
    y1 = anCorners[0][1]
    y2 = anCorners[1][1]
    topStripe = img[(x1-2):(x2+2), (y1-3):(y1-1)]
    leftStripe = img[(x1-3):(x1-1), (y1-2):(y2+2)]
    rightStripe = img[(x2+1):(x2+3), (y1-2):(y2+2)]
    bottomStrip = img[(x1-2):(x2+2), (y2+1):(y2+3)]
    meanSurround = np.mean([np.mean(topStripe), np.mean(leftStripe), 
                           np.mean(rightStripe), np.mean(bottomStrip)])
    meanSurround = np.round(meanSurround)
    bg = img.copy()
    filler = np.ones(bg[y1:y2, x1:x2].shape)*meanSurround
    bg[y1:y2, x1:x2] = filler
    showImage(bg, 'bg')
    
    # return background image
    return bg
    
def findBGauto(dataPath, spreadIndices):
    imList = os.listdir(dataPath)
    imList.sort()
    nFrames = len(imList)
    stepIndices = round(nFrames/100)
    if spreadIndices:
        indices = np.arange(0, nFrames, stepIndices)
    else:
        indices = np.arange(0,160)    
    
    # load first image for size
    im1Name = imList[0]
    im1Path = path.join(dataPath, im1Name)
    img1 = cv2.imread(im1Path,0)
    
    # initialise image stack
    imStack = np.empty((img1.shape[0], img1.shape[1], len(indices)), dtype='uint8')
    #imStack.fill(np.nan) # numpy nans are float!!!
    
    for myN, myIndex in enumerate(indices):
        imName = imList[myIndex]
        imPath = path.join(dataPath, imName)
        imStack[:,:,myN] = cv2.imread(imPath,0)
            
    # calculate median
    bg = np.median(imStack, axis=2)
    bg = np.uint8(bg)
    
    # return background image
    return bg
    
    
def trackTadpoleBGwarp(dataPath, imageCheckFolder, bg, thresh, warpMatrix, maxWidth, maxHeight):
    
    imList = os.listdir(dataPath)
    if len(imList) > 9999:
        imList = sorted(imList, key=lambda im: int(im[(im.rfind('-') + 1):im.rfind('.')]))
    else:
        imList.sort()
    
    # initialise variable to hold output
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    df = pd.DataFrame(d)
    
    os.chdir(imageCheckFolder)
    
    for index,im in enumerate(imList[0:(len(imList))]):
        if np.mod(index, 1000) == 0:
            print('currently at frame ', index)
        
        # load
        imPath = path.join(dataPath, im)
        img = cv2.imread(imPath,0)
        
        # subtract bg
        subtracted0 = bg - img
        subtracted1 = 255 - subtracted0
        # this is still pretty noisy in the tank locations; try opening to get rid of the noise
        kernel = np.ones((4,4),np.uint8)
        closing = cv2.morphologyEx(subtracted1, cv2.MORPH_CLOSE, kernel)        
        
        # warp
        warp = cv2.warpPerspective(img, warpMatrix, (maxWidth, maxHeight))
        
        # track
        try:
            currentPos, currentEllipse = tadpolePosAngle(closing, thresh) # i.e. track in the background-subtracted, UNWARPED image
            
            # warp the coordinates.... complicated. apparently the perspectiveTransform needs unflattened arrays, and
            # it does not work with only one point (so add a dummy point), and it does not work without specifying 
            # the data type... silly?!
            # for contour coordinate
            coordContour = np.array([((currentPos['tadpoleXcontour'], currentPos['tadpoleYcontour']), (5, 5))], dtype=np.float32)            
            coordContourWarped = cv2.perspectiveTransform(coordContour, warpMatrix)
            coordContourWarped = coordContourWarped[0][0]
            
            # for ellipse coordinate
            coordEllipse = np.array([((currentPos['tadpoleXellipse'], currentPos['tadpoleYellipse']), (5, 5))], dtype=np.float32)            
            coordEllipseWarped = cv2.perspectiveTransform(coordEllipse, warpMatrix)
            coordEllipseWarped = coordEllipseWarped[0][0]
            
            currentPos['tadpoleXcontour'], currentPos['tadpoleYcontour'] = coordContourWarped      
            currentPos['tadpoleXellipse'], currentPos['tadpoleYellipse'] = coordEllipseWarped
            
            df.loc[df.shape[0]] = currentPos
            
            # save image with coordinate to check in checkImagesPath
            cv2.circle(warp, (int(currentPos['tadpoleXcontour']), int(currentPos['tadpoleYcontour'])), 2, 230, 2)
            ellipse2 = list(currentEllipse)
            ellipse2[0] = tuple(coordEllipseWarped.tolist())
            ellipse2 = tuple(ellipse2)
            cv2.ellipse(warp, ellipse2, 20, 2)
        except:
            df.loc[df.shape[0]] = np.nan
            print('added nan value at index ', index)
        
        #imPath = path.join(checkImagesPath, 'im'+str(index)+'.tif')
        cv2.imwrite('im'+str(index)+'.tif', warp)
        
        del(img, subtracted0, subtracted1, closing)
        
    
        
        
    return df, warp.shape
    
    
def trackTadpoleBGwarpPadded(dataPath, imageCheckFolder, bg, thresh, warpMatrix, maxWidth, maxHeight, stage, tankSize, kernelSize):
    
    imList = os.listdir(dataPath)
    if len(imList) > 9999:
        imList = sorted(imList, key=lambda im: int(im[(im.rfind('-') + 1):im.rfind('.')]))
    else:
        imList.sort()
    
    # initialise variable to hold output
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    df = pd.DataFrame(d)
    
    os.chdir(imageCheckFolder)
    
    kernel = np.ones((kernelSize, kernelSize), np.uint8)
    
    for index,im in enumerate(imList[0:(len(imList))]):
        if np.mod(index, 1000) == 0:
            print('currently at frame ', index)
        #print('index: ', index)
        
        # load
        imPath = path.join(dataPath, im)
        img = cv2.imread(imPath,0)
        
        # subtract bg
        subtracted0 = bg - img
        subtracted1 = 255 - subtracted0
        # this is still pretty noisy in the tank locations; try opening to get rid of the noise
        closing = cv2.morphologyEx(subtracted1, cv2.MORPH_CLOSE, kernel)
        
        # try and get rid of pixels that became black due to bg subtraction
        # this is an issue especially for the small animals... which are never 
        # entirely black
        if stage < 50 or level1Index == 33:
            closing[(closing < 20)] = 255
        
        # add white padding: 2 pixels on each side
        padded = 255*np.ones((img.shape[0]+4, img.shape[1]+4), dtype='uint8')
        padded[2:-2, 2:-2] = closing
        
        # warp
        warp = cv2.warpPerspective(img, warpMatrix, (maxWidth, maxHeight))
        
        """
        # previous coordinates
        if df.shape[0] == 0:
            previousX = np.nan
            previousY = np.nan
        else:
            previousX = df.tadpoleXcontour[df.shape[0]-1]
            previousY = df.tadpoleYcontour[df.shape[0]-1]
        """
        
        # track
        try:
            currentPos, currentEllipse = tadpolePosAngle(padded, thresh) #, (previousX, previousY)) # i.e. track in the background-subtracted, UNWARPED image
            #else:
            #    currentPos, currentEllipse = tadpolePosAngleGaussian(padded, kernel)
            
            # warp the coordinates.... complicated. apparently the perspectiveTransform needs unflattened arrays, and
            # it does not work with only one point (so add a dummy point), and it does not work without specifying 
            # the data type... silly?!
            # for contour coordinate
            coordContour = np.array([((currentPos['tadpoleXcontour']-2, currentPos['tadpoleYcontour']-2), (5, 5))], dtype=np.float32)            
            coordContourWarped = cv2.perspectiveTransform(coordContour, warpMatrix)
            coordContourWarped = coordContourWarped[0][0]
            
            # for ellipse coordinate
            coordEllipse = np.array([((currentPos['tadpoleXellipse']-2, currentPos['tadpoleYellipse']-2), (5, 5))], dtype=np.float32)            
            coordEllipseWarped = cv2.perspectiveTransform(coordEllipse, warpMatrix)
            coordEllipseWarped = coordEllipseWarped[0][0]
            
            currentPos['tadpoleXcontour'], currentPos['tadpoleYcontour'] = coordContourWarped      
            currentPos['tadpoleXellipse'], currentPos['tadpoleYellipse'] = coordEllipseWarped
            
            df.loc[df.shape[0]] = currentPos
            
            # save image with coordinate to check in checkImagesPath
            cv2.circle(warp, (int(currentPos['tadpoleXcontour']), int(currentPos['tadpoleYcontour'])), 2, 230, 2)
            ellipse2 = list(currentEllipse)
            ellipse2[0] = tuple(coordEllipseWarped.tolist())
            ellipse2 = tuple(ellipse2)
            cv2.ellipse(warp, ellipse2, 20, 2)
            
        except:
            df.loc[df.shape[0]] = np.nan
            print('added nan value at index ', index)
        
        #imPath = path.join(checkImagesPath, 'im'+str(index)+'.tif')
        cv2.imwrite('im'+str(index)+'.tif', warp)
        
        del(img, subtracted0, subtracted1, closing)
        
    
        
        
    return df, warp.shape
    
    
def trackTadpoleWarpPadded(dataPath, imageCheckFolder, thresh, warpMatrix, maxWidth, maxHeight):
    
    imList = os.listdir(dataPath)
    if len(imList) > 9999:
        imList = sorted(imList, key=lambda im: int(im[(im.rfind('-') + 1):im.rfind('.')]))
    else:
        imList.sort()
    
    # initialise variable to hold output
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    df = pd.DataFrame(d)
    
    os.chdir(imageCheckFolder)
    
    for index,im in enumerate(imList[0:(len(imList))]):
        if np.mod(index, 1000) == 0:
            print('currently at frame ', index)
        #print('index: ', index)
        
        # load
        imPath = path.join(dataPath, im)
        img = cv2.imread(imPath,0)
        
        # add white padding: 2 pixels on each side
        padded = 255*np.ones((img.shape[0]+4, img.shape[1]+4), dtype='uint8')
        padded[2:-2, 2:-2] = img
        
        # warp
        warp = cv2.warpPerspective(img, warpMatrix, (maxWidth, maxHeight))
        
        # track
        try:
            currentPos, currentEllipse = tadpolePosAngle(padded, thresh) #, (previousX, previousY)) # i.e. track in the background-subtracted, UNWARPED image
            #else:
            #    currentPos, currentEllipse = tadpolePosAngleGaussian(padded, kernel)
            
            # warp the coordinates.... complicated. apparently the perspectiveTransform needs unflattened arrays, and
            # it does not work with only one point (so add a dummy point), and it does not work without specifying 
            # the data type... silly?!
            # for contour coordinate
            coordContour = np.array([((currentPos['tadpoleXcontour']-2, currentPos['tadpoleYcontour']-2), (5, 5))], dtype=np.float32)            
            coordContourWarped = cv2.perspectiveTransform(coordContour, warpMatrix)
            coordContourWarped = coordContourWarped[0][0]
            
            # for ellipse coordinate
            coordEllipse = np.array([((currentPos['tadpoleXellipse']-2, currentPos['tadpoleYellipse']-2), (5, 5))], dtype=np.float32)            
            coordEllipseWarped = cv2.perspectiveTransform(coordEllipse, warpMatrix)
            coordEllipseWarped = coordEllipseWarped[0][0]
            
            currentPos['tadpoleXcontour'], currentPos['tadpoleYcontour'] = coordContourWarped      
            currentPos['tadpoleXellipse'], currentPos['tadpoleYellipse'] = coordEllipseWarped
            
            df.loc[df.shape[0]] = currentPos
            
            # save image with coordinate to check in checkImagesPath
            cv2.circle(warp, (int(currentPos['tadpoleXcontour']), int(currentPos['tadpoleYcontour'])), 2, 230, 2)
            ellipse2 = list(currentEllipse)
            ellipse2[0] = tuple(coordEllipseWarped.tolist())
            ellipse2 = tuple(ellipse2)
            cv2.ellipse(warp, ellipse2, 20, 2)
            
        except:
            df.loc[df.shape[0]] = np.nan
            print('added nan value at index ', index)
        
        #imPath = path.join(checkImagesPath, 'im'+str(index)+'.tif')
        cv2.imwrite('im'+str(index)+'.tif', warp)
        
        del(img)
        
    return df, warp.shape
    
def nanProblemIndices(df):
    """
    function that given a panda dataframe finds indices of the rows contatining nans
    and indices where the contour and the ellipse coordinates are more than 5
    pixels apart
    """
    threshDist = 7 #pixels
    
    nanIndices = pd.isnull(df).any(1).nonzero()[0]
    
    xDiff = abs(df.tadpoleXcontour - df.tadpoleXellipse)
    yDiff = abs(df.tadpoleYcontour - df.tadpoleYellipse)
    
    xProblemIndices = xDiff[xDiff > threshDist]
    xProblemIndices = list(xProblemIndices.index)
    yProblemIndices = yDiff[yDiff > threshDist]
    yProblemIndices = list(yProblemIndices.index)
    problemIndices = xProblemIndices + yProblemIndices
    problemIndices = list(set(problemIndices)) # make it unique; will not be ordered
    problemIndices.sort() 
    
    return list(nanIndices), problemIndices
    
    
    
def reducePoints(df, epsilon, saveMyFig):
    
    # apply algorithm from rdp
    reduced = rdp([(x,y) for x,y in zip(df.tadpoleXcontour, df.tadpoleYcontour)], epsilon=epsilon)
    reducedX = [a[0] for a in reduced]
    reducedY = [a[1] for a in reduced]
    
    if saveMyFig:
        fig, ax = plt.subplots()
        fig.hold(True)
        df.plot(x='tadpoleXcontour', y='tadpoleYcontour', ax=ax, color='k')
        plt.plot(reducedX, reducedY, color='r')
        ax.axis('equal')
        plt.savefig('tmp.png', dpi=300)

    # Now find the matching points in the df to be able to get at the timing etc
    
    # make a df of the new points
    si = pd.DataFrame({'x':reducedX, 'y':reducedY})
    
    # make a new column in si for holding the index of the matching point into the df
    si['df_index'] = None
    
    # loop through si and then loop through df to find matching points
    startIndex = 0
    for si_i, si_row in si.iterrows():
        si_coords = (si_row['x'], si_row['y'])
        for df_i in np.arange(startIndex, len(df)):
            df_row = df.iloc[df_i,:]
            if si_coords == (df_row['tadpoleXcontour'], df_row['tadpoleYcontour']):
                si.loc[si_i, 'df_index'] = df_i
                startIndex = df_i
                break
    
    return si
    

def reconstructWarp(tankCorners):
    
    # calculate the stuff necessary for warping
    # the width of our new image
    (tl, tr, br, bl) = tankCorners  # should be the same order as Adrian's code - at least that is what I asked the user!
    widthA = np.sqrt(((br[0] - bl[0]) ** 2) + ((br[1] - bl[1]) ** 2))
    widthB = np.sqrt(((tr[0] - tl[0]) ** 2) + ((tr[1] - tl[1]) ** 2))
    
    # ...and now for the height of our new image
    heightA = np.sqrt(((tr[0] - br[0]) ** 2) + ((tr[1] - br[1]) ** 2))
    heightB = np.sqrt(((tl[0] - bl[0]) ** 2) + ((tl[1] - bl[1]) ** 2))
    
    # take the maximum of the width and height values to reach
    # our final dimensions
    maxWidth = max(int(widthA), int(widthB))
    maxHeight = max(int(heightA), int(heightB))

    # construct our destination points which will be used to
    # map the screen to a top-down, "birds eye" view
    dst = np.array([[0, 0], [maxWidth - 1, 0], [maxWidth - 1, maxHeight - 1], [0, maxHeight - 1]], dtype = "float32")

    # need corners again but different data type
    box2 = (tl, tr, br, bl)
    # and convert to the same data type
    box2 = np.float32(box2)

    # calculate the perspective transform matrix
    Matrix = cv2.getPerspectiveTransform(box2, dst)

    return Matrix, maxWidth, maxHeight
    
    
    
def makeBWthreshold(bg, param1, param2):
    #for tifs: param1 = 65, param2 = 25
    
    kernel = np.ones((2,2), np.uint8)    
    thBG = cv2.adaptiveThreshold(bg,255,  cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, param1, param2)
    closingBG = cv2.morphologyEx(thBG, cv2.MORPH_CLOSE, kernel)
    return closingBG
    
    
def trackTadpoleBWBGwarpPadded(dataPath, imageCheckFolder, bg, warpMatrix, maxWidth, maxHeight, stage, tankSize):
    
    imList = os.listdir(dataPath)
    if len(imList) > 9999:
        imList = sorted(imList, key=lambda im: int(im[(im.rfind('-') + 1):im.rfind('.')]))
    else:
        imList.sort()
    
    # initialise variable to hold output
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    df = pd.DataFrame(d)
    
    os.chdir(imageCheckFolder)
    
    kernel = np.ones((2, 2), np.uint8)
    param1 = 65
    param2 = 25
    closingTrue = True
    
    for index,im in enumerate(imList[0:(len(imList))]):
        if np.mod(index, 1000) == 0:
            print('currently at frame ', index)
        #print('index: ', index)
        
        # load
        imPath = path.join(dataPath, im)
        img = cv2.imread(imPath,0)
        
        # warp
        warp = cv2.warpPerspective(img, warpMatrix, (maxWidth, maxHeight))
        
        # track
        try:
            currentPos, currentEllipse = tadpolePosAngleGaussian(img, bg, kernel, param1, param2, closingTrue) #, (previousX, previousY)) # i.e. track in the background-subtracted, UNWARPED image
            #else:
            #    currentPos, currentEllipse = tadpolePosAngleGaussian(padded, kernel)
            
            # warp the coordinates.... complicated. apparently the perspectiveTransform needs unflattened arrays, and
            # it does not work with only one point (so add a dummy point), and it does not work without specifying 
            # the data type... silly?!
            # for contour coordinate
            coordContour = np.array([((currentPos['tadpoleXcontour']-2, currentPos['tadpoleYcontour']-2), (5, 5))], dtype=np.float32)            
            coordContourWarped = cv2.perspectiveTransform(coordContour, warpMatrix)
            coordContourWarped = coordContourWarped[0][0]
            
            # for ellipse coordinate
            coordEllipse = np.array([((currentPos['tadpoleXellipse']-2, currentPos['tadpoleYellipse']-2), (5, 5))], dtype=np.float32)            
            coordEllipseWarped = cv2.perspectiveTransform(coordEllipse, warpMatrix)
            coordEllipseWarped = coordEllipseWarped[0][0]
            
            currentPos['tadpoleXcontour'], currentPos['tadpoleYcontour'] = coordContourWarped      
            currentPos['tadpoleXellipse'], currentPos['tadpoleYellipse'] = coordEllipseWarped
            
            df.loc[df.shape[0]] = currentPos
            
            # save image with coordinate to check in checkImagesPath
            cv2.circle(warp, (int(currentPos['tadpoleXcontour']), int(currentPos['tadpoleYcontour'])), 2, 230, 2)
            ellipse2 = list(currentEllipse)
            ellipse2[0] = tuple(coordEllipseWarped.tolist())
            ellipse2 = tuple(ellipse2)
            cv2.ellipse(warp, ellipse2, 20, 2)
            
        except:
            df.loc[df.shape[0]] = np.nan
            print('added nan value at index ', index)
        
        #imPath = path.join(checkImagesPath, 'im'+str(index)+'.tif')
        cv2.imwrite('im'+str(index)+'.tif', warp)
        
        del(img)
        
    return df, warp.shape
                                                 
    