# -*- coding: utf-8 -*-
"""
Created on Thu Jan 19 13:32:27 2017
This script contains auxiliary functions necessary for tracking animals whose
videos were saved as avi files. these were very much empirically adjusted 
by what strategy worked.

@author: haenzi
"""

def findBGautoAvi(dataPath, imgShape):
    # open video reader
    cap = cv2.VideoCapture(dataPath)
    nFrame = 0
    nFrames = cap.get(cv2.CAP_PROP_FRAME_COUNT) 
    
    stepIndices = round(nFrames/100)
    indices = np.arange(0, nFrames, stepIndices)
    
    # initialise image stack
    imStack = np.empty((imgShape[0], imgShape[1], len(indices)), dtype='uint8')
    #imStack.fill(np.nan) # numpy nans are float!!!
    
    for myN, myIndex in enumerate(indices):
        cap.set(cv2.CAP_PROP_POS_FRAMES, myIndex)
        ret, frame = cap.read()
        imStack[:,:,myN] = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
            
    # calculate median
    bg = np.median(imStack, axis=2)
    bg = np.uint8(bg)
    
    cap.release()
    
    # return background image
    return bg
    
    
    
def trackAviBWbgPadded(dataPath, imageCheckFolder, bg, warpMatrix, maxWidth, maxHeight):
    
    cap = cv2.VideoCapture(dataPath)
    nFrames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    
    # initialise variable to hold output
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    df = pd.DataFrame(d)
    
    os.chdir(imageCheckFolder)
    
    kernel = np.ones((2, 2), np.uint8)
    param1 = 65
    param2 = 4
    closingTrue = False
        
    
    for index in np.arange(nFrames):
        if np.mod(index, 1000) == 0:
            print('currently at frame ', index)
        
        # load
        ret, frame = cap.read()   
        img = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        
        # warp
        warp = cv2.warpPerspective(img, warpMatrix, (maxWidth, maxHeight))
        warpedBG = cv2.warpPerspective(bg, warpMatrix, (maxWidth, maxHeight))
        
        # track
        try:
            currentPos, currentEllipse = tadpolePosAngleGaussian(warp, warpedBG, kernel, param1, param2, closingTrue) #, (previousX, previousY)) # i.e. track in the background-subtracted, UNWARPED image
            # take away the two pixels related to the padding
            currentPos['tadpoleXcontour'] -=2
            currentPos['tadpoleYcontour'] -=2
            currentPos['tadpoleXellipse'] -= 2
            currentPos['tadpoleYellipse'] -= 2
            
            df.loc[df.shape[0]] = currentPos
            
            # save image with coordinate to check in checkImagesPath
            cv2.circle(warp, (int(currentPos['tadpoleXcontour']), int(currentPos['tadpoleYcontour'])), 2, 230, 2)
            ellipse2 = ((currentPos['tadpoleXellipse'], currentPos['tadpoleYellipse']), currentEllipse[1], currentEllipse[2])
            cv2.ellipse(warp, ellipse2, 20, 2)
            
        except:
            df.loc[df.shape[0]] = np.nan
            print('added nan value at index ', index)
        
        #imPath = path.join(checkImagesPath, 'im'+str(index)+'.tif')
        cv2.imwrite('im'+str(index)+'.tif', warp)
        
        del(img)
        
    cap.release()
    
    return df, warp.shape
    
    
    
def trackAviWarpPadded(dataPath, imageCheckFolder, thresh, warpMatrix, maxWidth, maxHeight):
    
    cap = cv2.VideoCapture(dataPath)
    nFrames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    
    # initialise variable to hold output
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    df = pd.DataFrame(d)
    
    os.chdir(imageCheckFolder)
    
    for index in np.arange(nFrames):
        if np.mod(index, 1000) == 0:
            print('currently at frame ', index)
        #print('index: ', index)
        
        # load
        # load
        ret, frame = cap.read()   
        img = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        
        # warp
        warp = cv2.warpPerspective(img, warpMatrix, (maxWidth, maxHeight))        
        
        # add white padding: 2 pixels on each side
        padded = 255*np.ones((warp.shape[0]+4, warp.shape[1]+4), dtype='uint8')
        padded[2:-2, 2:-2] = warp
        
        # track
        try:
            currentPos, currentEllipse = tadpolePosAngle(padded, thresh)
            
            # take away the two pixels related to the padding
            currentPos['tadpoleXcontour'] -=2
            currentPos['tadpoleYcontour'] -=2
            currentPos['tadpoleXellipse'] -= 2
            currentPos['tadpoleYellipse'] -= 2
            
            df.loc[df.shape[0]] = currentPos
            
            # save image with coordinate to check in checkImagesPath
            cv2.circle(warp, (int(currentPos['tadpoleXcontour']), int(currentPos['tadpoleYcontour'])), 2, 230, 2)
            ellipse2 = ((currentPos['tadpoleXellipse'], currentPos['tadpoleYellipse']), currentEllipse[1], currentEllipse[2])
            cv2.ellipse(warp, ellipse2, 20, 2)
            
        except:
            df.loc[df.shape[0]] = np.nan
            print('added nan value at index ', index)
        
        #imPath = path.join(checkImagesPath, 'im'+str(index)+'.tif')
        cv2.imwrite('im'+str(index)+'.tif', warp)
        
        del(img)
    cap.release()
        
    return df, warp.shape


def trackAviWarpPaddedIR(dataPath, imageCheckFolder, thresh, warpMatrix, maxWidth, maxHeight, level1Index):
    
    cap = cv2.VideoCapture(dataPath)
    nFrames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    
    # initialise variable to hold output
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    df = pd.DataFrame(d)
    
    listBorder = [140, 142, 144]
    listLargeBorder = [134, 135, 136]
    
    os.chdir(imageCheckFolder)
    
    for index in np.arange(nFrames):
        if np.mod(index, 1000) == 0:
            print('currently at frame ', index)
    
        # load
        ret, frame = cap.read()   
        img = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        
        # warp
        warp = cv2.warpPerspective(img, warpMatrix, (maxWidth, maxHeight))        
        
        # add white padding: 2 pixels on each side
        padded = 255*np.ones((warp.shape[0]+4, warp.shape[1]+4), dtype='uint8')
        padded[2:-2, 2:-2] = warp
        
        # pad 2 more rows of pixels into the warp
        nPixelsToDelete = 3
        if level1Index in listBorder:
            nPixelsToDelete = 5
        elif level1Index in listLargeBorder:
            nPixelsToDelete = 7
        padded[2:(2+nPixelsToDelete),:] = 255
        padded[(-2-nPixelsToDelete):-2,:] = 255
        padded[:, 2:(2+nPixelsToDelete)] = 255
        padded[:, (-2-nPixelsToDelete):-2] = 255
        
        # track
        try:
            currentPos, currentEllipse = tadpolePosAngle(padded, thresh)
            
            # take away the two pixels related to the padding
            currentPos['tadpoleXcontour'] -=2
            currentPos['tadpoleYcontour'] -=2
            currentPos['tadpoleXellipse'] -= 2
            currentPos['tadpoleYellipse'] -= 2
            
            df.loc[df.shape[0]] = currentPos
            
            # save image with coordinate to check in checkImagesPath
            cv2.circle(warp, (int(currentPos['tadpoleXcontour']), int(currentPos['tadpoleYcontour'])), 2, 230, 2)
            ellipse2 = ((currentPos['tadpoleXellipse'], currentPos['tadpoleYellipse']), currentEllipse[1], currentEllipse[2])
            cv2.ellipse(warp, ellipse2, 20, 2)
            
        except:
            df.loc[df.shape[0]] = np.nan
            print('added nan value at index ', index)
        
        #imPath = path.join(checkImagesPath, 'im'+str(index)+'.tif')
        cv2.imwrite('im'+str(index)+'.tif', warp)
        
        del(img)
    cap.release()
        
    return df, warp.shape


    
def tadpolePosAngle3rd(img, thresh): 
    """
    function that takes a black-and-white image currently in the workspace, 
    thresholds it (simple binary threshold; threshold value as input), 
    calculates the contours, takes the third-largest contour and outputs
    its position and angle
    like tadpolePosAngle except it chooses the third-largest rather than the second-largest contour
    """
    
    # define output dict
    d = {k : [] for k in ['tadpoleXcontour', 'tadpoleYcontour', 'tadpoleXellipse', 'tadpoleYellipse','ellipseAngle']}
    
    # threshold
    ret,th1 = cv2.threshold(img,thresh,255,cv2.THRESH_BINARY)
    
    # find contours
    im2, contours, hierarchy = cv2.findContours(th1,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
    
    # make a list of the contour areas
    cArea = [cv2.contourArea(c) for c in contours]
    
    contourChoice = 3
    try:
        tadpoleIndex = cArea.index(sorted(cArea)[-contourChoice])
        tadpoleContour = contours[tadpoleIndex]
        M = cv2.moments(tadpoleContour)
        d['tadpoleXcontour'] = int(M['m10']/M['m00'])
        d['tadpoleYcontour'] = int(M['m01']/M['m00'])
        
        #fit an ellipse to the tadpole
        ellipse = cv2.fitEllipse(tadpoleContour)
        d['tadpoleXellipse'] = ellipse[0][0]
        d['tadpoleYellipse'] = ellipse[0][1]
        d['ellipseAngle'] = ellipse[-1]
    except: # if it does not work (most likely because there were too few points) try again with higher thresh
        ret,th1 = cv2.threshold(img,thresh+40,255,cv2.THRESH_BINARY)
        im2, contours, hierarchy = cv2.findContours(th1,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
        cArea = [cv2.contourArea(c) for c in contours]
        tadpoleIndex = cArea.index(sorted(cArea)[-contourChoice])
        tadpoleContour = contours[tadpoleIndex]
        M = cv2.moments(tadpoleContour)
        d['tadpoleXcontour'] = int(M['m10']/M['m00'])
        d['tadpoleYcontour'] = int(M['m01']/M['m00'])
        # fit ellipse
        ellipse = cv2.fitEllipse(tadpoleContour)
        d['tadpoleXellipse'] = ellipse[0][0]
        d['tadpoleYellipse'] = ellipse[0][1]
        d['ellipseAngle'] = ellipse[-1]
            
    return d, ellipse
