# -*- coding: utf-8 -*-
"""
Created on Fri Feb 24 15:15:49 2017
This script compares the animals swimming in the standard condition, with a cold light source
illuminating the tank, with with the same animals swimming with 4 IR lamps illuminating
the tank.


@author: haenzi
"""

# run prerequisites
import numpy as np
from matplotlib import pyplot as plt
import os
from os import path
import pandas as pd
import statsmodels.api as sm
import datetime as dt
from scipy import stats

runfile('D:/SH data/thigmo/python scripts/openCVtutorialUtils.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/newThigmoTrackingFunctions.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/plottingUtilities.py', wdir='D:/SH data/thigmo/python scripts')

# hardcoded folder locations
thigmoPath = 'D:\\SH data\\thigmo'
baseImageCheck = 'D:\\SH data\\thigmo\\analysis\\trackingCheck'
readDataPath = 'D:\\SH data\\thigmo\\analysis'
baseGroupPlots = 'D:\\SH data\\thigmo\\analysis\\groupPlots'

# read level1 file
os.chdir(thigmoPath)
level1 = pd.read_csv('thigmoOverviewForPandas.csv')
level1 = level1.set_index('pythonIndex')
corrections = np.load('corrections.npy').item()

os.chdir(readDataPath)

# identify the indices of the data taken in infrared
IRindices = np.where(level1.light == 'IR')[0]

# check whether there are animals in there which need to be excluded (tracking did not work)
# and remove those from the IRindices
if np.nansum(level1.exclude[IRindices]) > 0:
    removeIndices = [x for x in np.arange(len(IRindices)) if level1.exclude[IRindices[x]]==1]
    IRindices = np.delete(IRindices, removeIndices)

# find the animal IDs
IRIDs = [level1.animalID[IRindices[x]] for x in np.arange(len(IRindices))]

# find the level1indices of the animals in the normal light
normalLightIndices = []
for x in IRIDs:
    possibleIndices = np.where(level1.animalID == x)[0]
    normalLightIndices.append(possibleIndices[0]) #always the first one because IR data added to the end of level1

#initialise
stageList = np.zeros((len(IRIDs),1))
level1IndicesList = []
level1Indices = np.zeros((len(IRIDs),2))
#level1Indices[:,1] = IRindices
#distanceDistributions = []
cdfX = []
cdfY = []
proportionNearWall = np.zeros((len(IRIDs),2))

#set parameters outside the loop
nearWallThresh = 15

os.chdir(readDataPath)

# loop through selected data
for animalInd, myAnimal in enumerate(IRIDs):
    
    bothIndices = np.where(level1.animalID == myAnimal)[0]
    lightIndex = bothIndices[0]
    IRindex = bothIndices[1]
    
    try:
        dfLight = pd.read_pickle(level1.Filename[lightIndex] + '_df.pkl')
        reducedLight = pd.read_pickle(level1.Filename[lightIndex] + '_reduced.pkl')
        dfIR = pd.read_pickle(level1.Filename[IRindex] + '_df.pkl')
        reducedIR = pd.read_pickle(level1.Filename[IRindex] + '_reduced.pkl')
    except:
        print('animal ', myAnimal, ' has no pickle file')
        continue
    
    # exclude if the animal did not go at least once across the tank
    tankSizeMM = level1.tankSize[lightIndex]*10 #need a factor of 10 because the tank size is in cm and the dist in mm
    
    if sum(reducedLight.segDist.dropna()) < tankSizeMM or sum(reducedIR.segDist.dropna()) < tankSizeMM:
        print('animal ', myAnimal, ' not moving enough')
        continue
    
    stageList[animalInd,0] = level1.stage[lightIndex]
    level1Indices[animalInd,0] = lightIndex
    level1Indices[animalInd,1] = IRindex
    
    # proportion near wall
    proportionNearWall[animalInd,0] = sum(dfLight.distToEdge < nearWallThresh)/dfLight.shape[0]
    proportionNearWall[animalInd,1] = sum(dfIR.distToEdge < nearWallThresh)/dfIR.shape[0]
    
    # cdf per animal
    ecdfLight = sm.distributions.ECDF(dfLight.distToEdge)
    myMin = min(dfLight.distToEdge)
    myMax = max(dfLight.distToEdge)
    xLight = np.linspace(myMin, myMax, num=(myMax-myMin)*10)
    ecdfIR = sm.distributions.ECDF(dfIR.distToEdge)
    myMin = min(dfIR.distToEdge)
    myMax = max(dfIR.distToEdge)
    xIR = np.linspace(myMin, myMax, num=(myMax-myMin)*10)    
    
    cdfX.append([xLight, xIR])
    cdfY.append([ecdfLight(xLight), ecdfIR(xIR)])
    
    
# make folder to save them: today's date
# get or make folder to put the plots
now = dt.datetime.now()
todaysDate = now.strftime('%Y_%m_%d')
groupPlotsFolder = path.join(baseGroupPlots, todaysDate)
if not path.exists(groupPlotsFolder):
    os.mkdir(groupPlotsFolder) 
IRFolder = path.join(groupPlotsFolder, 'IRComparisons')
if not path.exists(IRFolder):
    os.mkdir(IRFolder)
os.chdir(IRFolder)

# cdf
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(cdfX)):
    ax.plot(cdfX[myInd][0], cdfY[myInd][0], c='k')
    ax.plot(cdfX[myInd][1], cdfY[myInd][1], c='r')
ax.set_xlabel('distance from nearest wall (mm)', fontsize=14)
ax.set_ylabel('cdf', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.legend(['light', 'IR'], loc='lower right', frameon=False)
plt.tight_layout()
plt.savefig('cdf_IR.png', dpi=300)
plt.savefig('cdf_IR.pdf')
plt.close()

# make a combined line and boxplot for the proportion near the wall
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(proportionNearWall)):
    ax.plot([1.25, 1.75], [proportionNearWall[myInd,0], proportionNearWall[myInd, 1]], c='k', linewidth = 0.5)
ax.set_ylabel('proportion near wall', fontsize=14)
ax.boxplot([proportionNearWall[:,0], proportionNearWall[:,1]],
           boxprops = {'color': 'black', 'linestyle': '-'}, 
            whiskerprops = {'color': 'black', 'linestyle': '-'}, 
            flierprops = {'color': 'black'})
ax.set_xticks([1,2])
ax.set_xticklabels(['light','IR'])
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
ax.set_xlim([0.8, 2.2])
ax.set_ylim(0,1)        
plt.tight_layout()
plt.savefig('IRCompPropBoxAndLine.png', dpi=300)
plt.savefig('IRCompPropBoxAndLine.pdf')



# statistics: are light and IR significantly different? ---------------------------

# proportion near the wall
w, p = stats.shapiro(proportionNearWall[:,0])
print('proportion near the wall, light: w = %g, p = %g' %(w,p)) # significant 
w, p = stats.shapiro(proportionNearWall[:,1])
print('proportion near the wall, IR: w = %g, p = %g' %(w,p)) #significant
wil, p = stats.wilcoxon(proportionNearWall[:,0], proportionNearWall[:,1])
print('proportion near the wall, light vs. IR: wil = %g, p = %g' %(wil,p)) #ns