# -*- coding: utf-8 -*-
"""
Created on Wed Feb  1 15:35:25 2017
This script compares the animals swimming in the standard condition, with white
paper surrounding the tank, with the same animals swimming with black paper
surrounding the tank.

@author: haenzi
"""

# run prerequisites
import numpy as np
from matplotlib import pyplot as plt
import os
from os import path
import pandas as pd
import statsmodels.api as sm
import datetime as dt
from scipy import stats

runfile('D:/SH data/thigmo/python scripts/openCVtutorialUtils.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/newThigmoTrackingFunctions.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/plottingUtilities.py', wdir='D:/SH data/thigmo/python scripts')

# hardcoded folder locations
thigmoPath = 'D:\\SH data\\thigmo'
baseImageCheck = 'D:\\SH data\\thigmo\\analysis\\trackingCheck'
readDataPath = 'D:\\SH data\\thigmo\\analysis'
baseGroupPlots = 'D:\\SH data\\thigmo\\analysis\\groupPlots'

# read level1 file
os.chdir(thigmoPath)
level1 = pd.read_csv('thigmoOverviewForPandas.csv')
level1 = level1.set_index('pythonIndex')
corrections = np.load('corrections.npy').item()

os.chdir(readDataPath)

# identify the indices of the animals that should be plotted
# for black-and-white comparison need to go from level1Index 7 up to and including 28
# or animalIDs 8 up to and including 18
bwIndices = np.arange(7,29)

# loop through those selected data and calculate the desired values

#initialise
stageList = []
level1Indices = []
animalIDs = []
cdfX = []
cdfY = []
proportionNearWall = []

#set parameters outside the loop
nearWallThresh = 15

os.chdir(readDataPath)

# loop through selected data
for myIndex in bwIndices:

    if level1.exclude[myIndex] == 1 or level1.trackingCheck[myIndex] != 'ok':
        continue
    
    try:
        df = pd.read_pickle(level1.Filename[myIndex] + '_df.pkl')
        reduced = pd.read_pickle(level1.Filename[myIndex] + '_reduced.pkl')
    except:
        print('index ', myIndex, ' has no pickle file')
        continue
    
    # exclude if the animal did not go at least once across the tank
    tankSizeMM = level1.tankSize[myIndex]*10 #need a factor of 10 because the tank size is in cm and the dist in mm
    
    if sum(reduced.segDist.dropna()) < tankSizeMM:
        print('index ', myIndex, ' not moving enough')
        continue
    
    stageList.append(level1.stage[myIndex])
    level1Indices.append(myIndex)
    animalIDs.append(level1.animalID[myIndex])
    
    # calculate proportion near wall
    proportionNearWall.append(sum(df.distToEdge < nearWallThresh)/df.shape[0])
    
    # cdf per animal
    ecdf = sm.distributions.ECDF(df.distToEdge)
    myMin = min(df.distToEdge)
    myMax = max(df.distToEdge)
    x1 = np.linspace(myMin, myMax, num=(myMax-myMin)*10)
    cdfX.append(x1)
    cdfY.append(ecdf(x1))
    
# because level1Index 23 does not survive the distance criterion (does not move
    #enough) I also have to get rid of level1Index 24, which is the same animal
    # (animalID 16) but with black walls
deleteIndex = np.where(np.array(level1Indices) == 24)[0]
level1Indices.pop(deleteIndex)
stageList.pop(deleteIndex)
animalIDs.pop(deleteIndex)
cdfX.pop(deleteIndex)
cdfY.pop(deleteIndex)
proportionNearWall.pop(deleteIndex)
    
# make folder to save them: today's date
# get or make folder to put the plots
now = dt.datetime.now()
todaysDate = now.strftime('%Y_%m_%d')
groupPlotsFolder = path.join(baseGroupPlots, todaysDate)
if not path.exists(groupPlotsFolder):
    os.mkdir(groupPlotsFolder) 
bwFolder = path.join(groupPlotsFolder, 'bwComparisons')
if not path.exists(bwFolder):
    os.mkdir(bwFolder)
os.chdir(bwFolder)

# cdf
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(cdfX)):
    if 'black' in level1.Filename[level1Indices[myInd]]:
        ax.plot(cdfX[myInd], cdfY[myInd], c='k')
    else:
        ax.plot(cdfX[myInd], cdfY[myInd], c=[0.7, 0.7, 0.7])
ax.set_xlabel('distance from wall (mm)', fontsize=14)
ax.set_ylabel('cdf', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.legend(['white', 'black'], loc='lower right', frameon=False)
plt.tight_layout()
plt.savefig('cdf_bw.png', dpi=300)
plt.savefig('cdf_bw.pdf')
plt.close()

uniqueAnimalIDs = np.unique(level1.animalID[level1Indices])

# for the black and white videos of the same animal, in level1 white always
# comes before black                       

# get the data in more useful format: array with one row per animal
proportionNearWallBW = np.zeros((len(uniqueAnimalIDs), 2))

for listInd, animalID in enumerate(uniqueAnimalIDs):
    myIndices = np.where(np.array(animalIDs) == animalID)[0]
    firstLevel1Index = level1Indices[myIndices[0]]
    if 'black' in level1.Filename[firstLevel1Index]:
        print('black is first')
        blackIndex = 0
        whiteIndex = 1
    else:
        print('white is first')        
        blackIndex = 1
        whiteIndex = 0
    proportionNearWallBW[listInd,0] = proportionNearWall[myIndices[blackIndex]]
    proportionNearWallBW[listInd,1] = proportionNearWall[myIndices[whiteIndex]]
                                        
# plot proportion near wall in black and white
fig, ax = plt.subplots()
ax.hold(True)
for listInd, animalID in enumerate(uniqueAnimalIDs):
    ax.plot([1, 2], proportionNearWallBW[listInd,:], c='k')
ax.set_xticks([1,2])
ax.set_xticklabels(['black', 'white'])
ax.set_ylabel('proportion near wall', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
ax.set_xlim([0.8, 2.2])
ax.set_ylim(0,1)
plt.tight_layout()
plt.savefig('proportionNearWallBW.png', dpi=300)
plt.savefig('proportionNearWAllBW.pdf')
plt.close()

#make a combined line and boxplot for the proportion near the wall
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(proportionNearWallBW)):
    ax.plot([1.25, 1.75], [proportionNearWallBW[myInd,0], proportionNearWallBW[myInd,1]], c='k', linewidth=0.5)
ax.set_ylabel('proportion near the wall', fontsize=14)
ax.boxplot([proportionNearWallBW[:,0], proportionNearWallBW[:,1]], 
           boxprops = {'color': 'black', 'linestyle': '-'}, 
            whiskerprops = {'color': 'black', 'linestyle': '-'}, 
            flierprops = {'color': 'black'})
ax.set_xticks([1,2])
ax.set_xticklabels(['black','white'])
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
ax.set_xlim([0.8, 2.2])
ax.set_ylim(0,1)        
plt.tight_layout()
plt.savefig('bwCompPropBoxAndLine.png', dpi=300)
plt.savefig('bwCompPropBoxAndLine.pdf')

# statistics: are b and w significantly different? ---------------------------

# proportion near wall
w, p = stats.shapiro(proportionNearWallBW[:,0])
print("proportion near wall, black:   w = %g  p = %g" % (w, p)) # ns
w, p = stats.shapiro(proportionNearWallBW[:,1])
print("proportion near wall, white:   w = %g  p = %g" % (w, p)) # ns
# therefore do a paired ttest
t, p = stats.ttest_rel(proportionNearWallBW[:,0], proportionNearWallBW[:,1])
print('proportion near wall, black vs white: t = %g, p = %g' %(t,p)) # ns 
