# -*- coding: utf-8 -*-
"""
Created on Mon Nov 21 11:24:17 2016
This script contains functions to ask the user to click on the four corners of the 
tank given an image; these are used by the tankCornersLoop script.

@author: haenzi
"""
import numpy as np
import cv2

tankCorners = []


def clickTank(event, x, y, flags, param):
    # grab references to the global variables
    global tankCorners

    # check to see if the left mouse button was released

    if event == cv2.EVENT_LBUTTONUP:
        # record the ending (x, y) coordinates and add that point to the list
        tankCorners.append((x, y))


def findCorners(img, tankSize):
    """
    function called findCorners that takes a black-and-white image from the
    workspace and asks the user to click on the four corners of the tank
    order: top left - top right - bottom right - bottom left
    as an output the function passes the matrix which is needed to warp the 
    original image to only the tank
    """    

    # load the image, clone it, and setup the mouse callback function
    # because my screen is less than 1200 in both directions...
    if img.shape[0] > 1050:
        cv2.namedWindow("click the 4 corners of the tank - tl, tr, br, bl", cv2.WINDOW_NORMAL)
    else:
        cv2.namedWindow("click the 4 corners of the tank - tl, tr, br, bl")
    cv2.setMouseCallback("click the 4 corners of the tank - tl, tr, br, bl", clickTank)
    
    if (tankSize == 7) or (tankSize == 19):
        th3 = cv2.adaptiveThreshold(img, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C,
                                cv2.THRESH_BINARY, 45, 5)
        clone = th3.copy()
    else:
        th3 = img.copy()
        th3[th3 != 255] -= 40
        th3[th3 < 0] = 0
        clone = img.copy()
        

    while len(tankCorners) < 4:
        # display the image and wait for a keypress
        cv2.imshow("click the 4 corners of the tank - tl, tr, br, bl", th3)
        key = cv2.waitKey(1) & 0xFF

        # if the 'c' key is pressed, break from the loop
        if key == ord("c"):
            break

    # close that window
    cv2.destroyAllWindows()

    # show the corners
    if len(tankCorners) == 4:
        for ind in range(4):
            try:  # somehow it sometimes needs one number and sometimes three for the colour...strange
                cv2.circle(clone, tankCorners[ind], 2, (200), 2)
            except:
                cv2.circle(clone, tankCorners[ind], 2, (200, 200, 200), 2)
        cv2.imshow("ROI", clone)
        cv2.waitKey(0)

    # close all open windows
    cv2.destroyAllWindows()

    # calculate the stuff necessary for warping
    # the width of our new image
    (tl, tr, br, bl) = tankCorners  # should be the same order as Adrian's code - at least that is what I asked the user!
    widthA = np.sqrt(((br[0] - bl[0]) ** 2) + ((br[1] - bl[1]) ** 2))
    widthB = np.sqrt(((tr[0] - tl[0]) ** 2) + ((tr[1] - tl[1]) ** 2))
    
    # ...and now for the height of our new image
    heightA = np.sqrt(((tr[0] - br[0]) ** 2) + ((tr[1] - br[1]) ** 2))
    heightB = np.sqrt(((tl[0] - bl[0]) ** 2) + ((tl[1] - bl[1]) ** 2))
    
    # take the maximum of the width and height values to reach
    # our final dimensions
    maxWidth = max(int(widthA), int(widthB))
    maxHeight = max(int(heightA), int(heightB))

    # construct our destination points which will be used to
    # map the screen to a top-down, "birds eye" view
    dst = np.array([[0, 0], [maxWidth - 1, 0], [maxWidth - 1, maxHeight - 1], [0, maxHeight - 1]], dtype = "float32")

    # need corners again but different data type
    box2 = (tl, tr, br, bl)
    # and convert to the same data type
    box2 = np.float32(box2)

    # calculate the perspective transform matrix
    Matrix = cv2.getPerspectiveTransform(box2, dst)

    return Matrix, maxWidth, maxHeight, box2