# script to plot parameters from different animals


# run prerequisites
import numpy as np
from matplotlib import pyplot as plt
import os
from os import path
import pandas as pd
import statsmodels.api as sm
import datetime as dt
from matplotlib import cm
from scipy import stats

runfile('D:/SH data/thigmo/python scripts/openCVtutorialUtils.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/newThigmoTrackingFunctions.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/plottingUtilities.py', wdir='D:/SH data/thigmo/python scripts')

# hardcoded folder locations
thigmoPath = 'D:\\SH data\\thigmo'
baseImageCheck = 'D:\\SH data\\thigmo\\analysis\\trackingCheck'
readDataPath = 'D:\\SH data\\thigmo\\analysis'
baseGroupPlots = 'D:\\SH data\\thigmo\\analysis\\groupPlots'

# read level1 file
os.chdir(thigmoPath)
level1 = pd.read_csv('thigmoOverviewForPandas.csv')
level1 = level1.set_index('pythonIndex')
corrections = np.load('corrections.npy').item()

os.chdir(readDataPath)

# identify the indices of the animals that should be plotted
# start with all primary ones
primaryIndices = np.where(level1.primary == 1)[0]

# loop through those selected data and calculate the desired values

#initialise
stageList = []
proportionNearWall = []
level1Indices = []
cdfX = []
cdfY = []
distCoveredRed = [] # from the reduced trajectory
distCoveredBLRed = []
propWallQuartered = []
redCompressed = []

#set parameters outside the loop
nearWallThresh = 15 

os.chdir(readDataPath)

# loop through selected data
for myIndex in primaryIndices:

    # exclude those videos where the tracking did not work
    if level1.exclude[myIndex] == 1 or level1.trackingCheck[myIndex] != 'ok':
        continue
    
    # read in the tracked data
    try:
        #df: the dataframe containing all frames        
        df = pd.read_pickle(level1.Filename[myIndex] + '_df.pkl')
        #reduced: the dataframe only containing those coordinates that persisted after simplifying the original trajectory using the rdp algorithm
        reduced = pd.read_pickle(level1.Filename[myIndex] + '_reduced.pkl')
    except:
        print('index ', myIndex, ' has no pickle file')
        continue
    
    # exclude if the animal did not go at least once across the tank
    tankSizeMM = level1.tankSize[myIndex]*10 #need a factor of 10 because the tank size is in cm and the dist in mm
    if sum(reduced.segDist.dropna()) < tankSizeMM:
        print('index ', myIndex, ' not moving enough')
        continue
    
    # record the animal's stage and the index into level1
    stageList.append(level1.stage[myIndex])
    level1Indices.append(myIndex)
    
    # calculate proportion of time the animal spent near wall
    proportionNearWall.append(sum(df.distToEdge < nearWallThresh)/df.shape[0])
    
    # separate distance to edge for 4 quarters of the video
    quarterTimeIndex = round(len(df)/4)
    distToEdge1 = df.distToEdge[:quarterTimeIndex]
    distToEdge2 = df.distToEdge[quarterTimeIndex:(2*quarterTimeIndex)]
    distToEdge3 = df.distToEdge[(2*quarterTimeIndex):(3*quarterTimeIndex)]
    distToEdge4 = df.distToEdge[(3*quarterTimeIndex):]
    
    # calculate proportion near the wall separately for every quarter of the video
    propWallQuartered.append([sum(distToEdge1 < nearWallThresh)/len(distToEdge1),
                              sum(distToEdge2 < nearWallThresh)/len(distToEdge2), 
                            sum(distToEdge3 < nearWallThresh)/len(distToEdge3), 
                            sum(distToEdge4 < nearWallThresh)/len(distToEdge4)])
                          
    # calculate total distance covered
    distCoveredRed.append(sum(reduced.segDist.dropna()))
    
    # calculate distance covered in body lengths
    currentBLmm = level1.length[myIndex]*10 #multiply by 10 to get it in mm
    distCoveredBLRed.append(sum(reduced.segDist.dropna())/currentBLmm)
    
    # cdf per animal
    ecdf = sm.distributions.ECDF(df.distToEdge)
    myMin = min(df.distToEdge)
    myMax = max(df.distToEdge)
    x1 = np.linspace(myMin, myMax, num=(myMax-myMin)*10)
    cdfX.append(x1)
    cdfY.append(ecdf(x1))
        
    # calculate how much the trajectory was compressed by reducing it with rdp
    redCompressed.append(1 - (len(reduced)/len(df)))

# and now make plots

# make folder to save them: today's date
# get or make folder to put the plots
now = dt.datetime.now()
todaysDate = now.strftime('%Y_%m_%d')
groupPlotsFolder = path.join(baseGroupPlots, todaysDate)
if not path.exists(groupPlotsFolder):
    os.mkdir(groupPlotsFolder) 
os.chdir(groupPlotsFolder)

# proportion near wall vs stage
tankSizeCM = 19
tankSizeMM = tankSizeCM * 10
areaNearWall = nearWallThresh*tankSizeMM*2 + nearWallThresh*(tankSizeMM - 2*nearWallThresh)*2
areaProportion = areaNearWall/(tankSizeMM**2)
scatterSara(stageList, proportionNearWall, 'stage', 
'proportion within ' + str(nearWallThresh) + 'mm of the wall', 'stageVsPropNearWall')
print('proportion of near-wall area vs. whole tank: ', areaProportion)

# proportion near wall vs. stage with area proportion
scatterSaraHorLine(stageList, proportionNearWall, 'stage', 'proportion within ' + str(nearWallThresh) + 'mm of the wall',
                   'stageVsPropNearWallWithArea', areaProportion)

# colour-coded cdfs
stageForColour = np.arange(min(stageList), max(stageList) + 1) #np.unique(stageList)
myColours = stageForColour - min(stageForColour) + 2
myColours = myColours/max(myColours)
#to invert the color map:
myColours = myColours[::-1]

fig, ax = plt.subplots()
ax.hold(True)
myColourStuff = []
for myInd in np.arange(len(cdfX)):
    colourIndex = np.where(stageForColour == stageList[myInd])[0]
    myColourStuff.append(myColours[colourIndex][0])
    ax.plot(cdfX[myInd], cdfY[myInd], c=cm.inferno(myColours[colourIndex][0]))
ax.set_xlabel('distance from wall (mm)', fontsize=14)
ax.set_ylabel('cdf', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('cdf.png', dpi=300)
plt.savefig('cdf.pdf')
plt.plot([nearWallThresh, nearWallThresh], [0,1], '--', c='k')
plt.savefig('cdfWithThresh.png', dpi=300)
plt.savefig('cdfWithThresh.pdf')

# need a legend...make it in two halves, otherwise some of it is hidden
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(round(len(stageForColour)/2)): #len(stageForColour)):
    ax.plot(np.arange(3), np.arange(3), c=cm.inferno(myColours[myInd]))
ax.legend([str(st) for st in stageForColour[:round(len(stageForColour)/2)]])
plt.tight_layout()
plt.savefig('cdfLegendFirstHalf.png', dpi=300)
plt.savefig('cdfLegendFirstHalf.pdf')

fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(round(len(stageForColour)/2), len(stageForColour)):
    ax.plot(np.arange(3), np.arange(3), c=cm.inferno(myColours[myInd]))
ax.legend([str(st) for st in stageForColour[round(len(stageForColour)/2):]])
plt.tight_layout()
plt.savefig('cdfLegendSecondHalf.png', dpi=300)
plt.savefig('cdfLegendSecondHalf.pdf')


# stuff separately for every quarter of the video ###########################
propWallQuartArray = np.array(propWallQuartered)

# proportion near the wall
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(propWallQuartered)):
    colourIndex = np.where(stageForColour == stageList[myInd])[0]
    ax.plot([1,2,3,4], propWallQuartered[myInd], c=cm.inferno(myColours[colourIndex][0]), 
            linewidth=0.4)
ax.plot([1,2,3,4], np.median(propWallQuartArray, axis=0), 'o-', c='k', linewidth=3, markersize=10)
ax.set_xticks([1,2,3,4])
ax.set_xticklabels(['1st', '2nd', '3rd', '4th'])
minX = 0.6
maxX = 4.4
ax.plot([minX, maxX], [areaProportion, areaProportion], '--', c='k')
ax.set_xlim(minX, maxX)
ax.set_xlabel('quarter of the video', fontsize=14)
ax.set_ylabel('proportion within ' + str(nearWallThresh) + 'mm of the wall', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('propNearWallQuarteredWithThresh.png', dpi=300)
plt.savefig('propNearWallQuarteredWithThresh.pdf')

# proportion near wall boxplot
fig, ax = plt.subplots()
ax.hold(True)
ax.boxplot(proportionNearWall, boxprops = {'color': 'black', 'linestyle': '-'}, 
            whiskerprops = {'color': 'black', 'linestyle': '-'}, 
            flierprops = {'color': 'black'})
minX, maxX = ax.get_xlim()
ax.plot([minX, maxX], [areaProportion, areaProportion], '--', c='k')
ax.set_xlim(minX, maxX)
ax.set_ylabel('proportion within ' + str(nearWallThresh) + 'mm of the wall', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('propNearWallWithThreshBoxplot.png', dpi=300)
plt.savefig('propNearWallWithThreshBoxplot.pdf')

stdev = 1/100
jitteredX = np.ones(len(proportionNearWall)) + np.random.randn(len(proportionNearWall)) * stdev
scatterColorCodedHorLine(jitteredX, proportionNearWall, 'jitter', 'proportion near wall', stageList, 
                   stageForColour, 'jitteredPropNearWall', areaProportion)


# ------------------ distance covered based on reduced/simplified trajectory -------------------------------------

# distance covered vs. stage
scatterSara(stageList, np.array(distCoveredRed)/1000, 'stage', 'total distance covered (m)', 'stageVsTotalDistRed')

# distance covered vs. proportion near wall
scatterColorCoded(np.array(distCoveredRed)/1000, proportionNearWall, 'total distance covered (m)', 
                  'proportion near wall', stageList, stageForColour, 'totalDistRedVsPropNearWall')

# proportion with area proportion
scatterColorCodedHorLine(np.array(distCoveredRed)/1000, proportionNearWall, 'total distance covered (m)', 
                  'proportion near wall', stageList, stageForColour, 'totalDistRedVsPropNearWallWithThresh', areaProportion)

                  
# manually doing some stats... ####################################################
                  
# is the measure of wall following normally distributed?
                  
w, p = stats.shapiro(proportionNearWall)
print('prop near wall: w = %g, p = %g' %(w,p)) #significant

rho, p = stats.spearmanr(stageList, proportionNearWall)
print('Spearman R for stage vs. proportion near wall: rho = %g, p = %g' %(rho,p)) # significant:

# total distance covered (reduced) against measures of wall following
rho, p = stats.spearmanr(distCoveredRed, proportionNearWall)
print('Spearman R for distance covered (red) vs. proportion near wall: rho = %g, p = %g' %(rho,p)) # ns
                  
# compare first and last quarter of video for the three measures of thigmotaxis
# read out first and alst quarter into lists, then arrays
propWall1stQuarter = np.ones((len(propWallQuartered)))
propWall2ndQuarter = np.ones((len(propWallQuartered)))
propWall3rdQuarter = np.ones((len(propWallQuartered)))
propWall4thQuarter = np.ones((len(propWallQuartered)))
for myInd in np.arange(len(propWallQuartered)):
    propWall1stQuarter[myInd] = propWallQuartered[myInd][0]
    propWall2ndQuarter[myInd] = propWallQuartered[myInd][1]    
    propWall3rdQuarter[myInd] = propWallQuartered[myInd][2]    
    propWall4thQuarter[myInd] = propWallQuartered[myInd][-1]
   
# proportion near wall
w, p = stats.shapiro(propWall1stQuarter)
print('prop near wall, 1st quarter: w = %g, p = %g' %(w,p)) #significant
w, p = stats.shapiro(propWall4thQuarter)
print('prop near wall, 4th quarter: w = %g, p = %g' %(w,p)) #significant
# friedman
fried, p = stats.friedmanchisquare(propWall1stQuarter, propWall2ndQuarter, propWall3rdQuarter, propWall4thQuarter)

