# -*- coding: utf-8 -*-
"""
Created on Wed Jan  4 15:41:55 2017
This script contains functions to make graphs on some aspects of the videos, both
for comparing parameters in different animals as well as to make graphs about
individual animals/videos (with the individualPlots function).

@author: haenzi
"""
from matplotlib import pyplot as plt
import numpy as np
from matplotlib import cm
import os
from os import path
import pandas as pd
import statsmodels.api as sm

# this script contains functions for plotting data from the wall following project

def scatterSara(x,y,xlabel,ylabel,fnToSave):
    
    fig,ax = plt.subplots()
    ax.scatter(x, y, c='k')
    ax.set_xlabel(xlabel, fontsize=14)
    ax.set_ylabel(ylabel, fontsize=14)
    
    if xlabel[:10] == 'proportion' or xlabel == 'KS statistic' or xlabel == 'one-sample KS statistic':
        ax.set_xlim([0,1])
    if ylabel[:10] == 'proportion' or ylabel == 'KS statistic' or ylabel == 'one-sample KS statistic':
        ax.set_ylim([0,1])
    
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        label.set_fontsize(14)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.tight_layout()
    plt.savefig(fnToSave+'.png', dpi=300)
    plt.savefig(fnToSave+'.pdf')
    
def scatterSaraHorLine(x,y,xlabel,ylabel,fnToSave, lineValue):
    
    fig,ax = plt.subplots()
    ax.scatter(x, y, c='k')
    ax.set_xlabel(xlabel, fontsize=14)
    ax.set_ylabel(ylabel, fontsize=14)
    minX, maxX = ax.get_xlim()
    ax.plot([minX, maxX], [lineValue, lineValue], '--', c='k')
    ax.set_xlim(minX, maxX)
    
    if xlabel[:10] == 'proportion' or xlabel == 'KS statistic' or xlabel == 'one-sample KS statistic':
        ax.set_xlim([0,1])
    if ylabel[:10] == 'proportion' or ylabel == 'KS statistic' or ylabel == 'one-sample KS statistic':
        ax.set_ylim([0,1])
    
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        label.set_fontsize(14)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.tight_layout()
    plt.savefig(fnToSave+'.png', dpi=300)
    plt.savefig(fnToSave+'.pdf')


    
def scatterColorCoded(x,y, xlabel, ylabel, stageList, stageForColour, fnToSave):
    #with stage color-coded:
    fig, ax = plt.subplots()
    ax.hold(True)
    for myInd in np.arange(len(x)):
        colourIndex = np.where(stageForColour == stageList[myInd])[0]
        #myColourStuff.append(myColours[colourIndex][0])
        ax.scatter(x[myInd], y[myInd], c=cm.inferno(myColours[colourIndex][0]), edgecolors='none')
    if xlabel[:10] == 'proportion' or xlabel == 'KS statistic' or xlabel == 'one-sample KS statistic':
        ax.set_xlim([0,1])
    if ylabel[:10] == 'proportion' or ylabel == 'KS statistic' or ylabel == 'one-sample KS statistic':
        ax.set_ylim([0,1])
        
    ax.set_xlabel(xlabel, fontsize=14)
    ax.set_ylabel(ylabel, fontsize=14)
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        label.set_fontsize(14)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.tight_layout()
    plt.savefig(fnToSave+'.png', dpi=300)
    plt.savefig(fnToSave+'.pdf')
    plt.close()
    
def scatterColorCodedHorLine(x,y, xlabel, ylabel, stageList, stageForColour, fnToSave, lineValue):
    #with stage color-coded:
    fig, ax = plt.subplots()
    ax.hold(True)
    for myInd in np.arange(len(x)):
        colourIndex = np.where(stageForColour == stageList[myInd])[0]
        #myColourStuff.append(myColours[colourIndex][0])
        ax.scatter(x[myInd], y[myInd], c=cm.inferno(myColours[colourIndex][0]), edgecolors='none')
    minX, maxX = ax.get_xlim()
    ax.plot([minX, maxX], [lineValue, lineValue], '--', c='k')
    ax.set_xlim(minX, maxX)
    
    if xlabel[:10] == 'proportion' or xlabel == 'KS statistic' or xlabel == 'one-sample KS statistic':
        ax.set_xlim([0,1])
    if ylabel[:10] == 'proportion' or ylabel == 'KS statistic' or ylabel == 'one-sample KS statistic':
        ax.set_ylim([0,1])
    #ax.set_xlim(0.5, 1.5)
    ax.set_xlabel(xlabel, fontsize=14)
    ax.set_ylabel(ylabel, fontsize=14)
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        label.set_fontsize(14)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.tight_layout()
    plt.savefig(fnToSave+'.png', dpi=300)
    plt.savefig(fnToSave+'.pdf')
    plt.close()

    
def lines2conditions(condition1, condition2, conditionLabels, ylabel, stageForColour, stageList, fnToSave):

    fig, ax = plt.subplots()
    ax.hold(True)
    for myInd in np.arange(len(condition1)):
        colourIndex = np.where(stageForColour == stageList[myInd])[0]
        ax.plot([1, 2], [condition1[myInd], condition2[myInd]], c=cm.inferno(myColours[colourIndex][0]))
    ax.set_xticks([1,2])
    ax.set_xticklabels(conditionLabels)
    ax.set_ylabel(ylabel, fontsize=14)
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        label.set_fontsize(14)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    ax.set_xlim([0.8, 2.2])
    if ylabel[:10] == 'proportion' or ylabel == 'KS statistic' or ylabel == 'one-sample KS statistic':
        ax.set_ylim([0,1])        
    plt.tight_layout()
    plt.savefig(fnToSave+'.png', dpi=300)
    plt.savefig(fnToSave+'.pdf')
    plt.close()
    
    
def individualPlots(level1, level1Index, corrections, thigmoPath, baseImageCheck, savePath, savePathOriginals, basePlots, timestampPath):

    print('working on index ', level1Index)
    print('filename: ', level1.Filename[level1Index])

    # load appropriate data frame
    os.chdir(savePathOriginals)
    df = pd.read_pickle(level1.Filename[level1Index] + '_original_df.pkl')
    # load reduced: reduced = pd.read_pickle(level1.Filename[level1Index] + '_reduced.pkl')
    
    # get or make folder to put the plots
    plotsCheckFolder = path.join(basePlots, level1.Filename[level1Index] + '_plots')
    if not path.exists(plotsCheckFolder):
        os.mkdir(plotsCheckFolder) 
    os.chdir(plotsCheckFolder)
    titleString = 'level1Index ' + str(level1Index)
    
    tankCorners = [[0, 0], [0, 0], [0, 0], [0, 0]]
    tankCorners[0][0] = level1.tank0x[level1Index]
    tankCorners[0][1] = level1.tank0y[level1Index]
    tankCorners[1][0] = level1.tank1x[level1Index]
    tankCorners[1][1] = level1.tank1y[level1Index]
    tankCorners[2][0] = level1.tank2x[level1Index]
    tankCorners[2][1] = level1.tank2y[level1Index]
    tankCorners[3][0] = level1.tank3x[level1Index]
    tankCorners[3][1] = level1.tank3y[level1Index]
    warpMatrix, maxWidth, maxHeight = reconstructWarp(tankCorners)
    
    # read out some parameters
    framerate = level1.framerate[level1Index]
    stage = level1.stage[level1Index]
    tankSize = level1.tankSize[level1Index] #this is in cm
    tankSizeMM = tankSize*10
    warpShape = (maxHeight, maxWidth)
    
    # plot trajectory WITHOUT porential corrections
    plt.plot(df.tadpoleXcontour, df.tadpoleYcontour, color='k')
    plt.hold(True)
    plt.axes().set_aspect('equal')
    plt.ylim(maxHeight+10, -10)
    plt.xlim(-10, maxWidth+10)
    plt.plot([0, maxWidth, maxWidth, 0, 0], [0, 0, maxHeight, maxHeight, 0], color='k')
    plt.axis('off')
    plt.title('trajectory no correction, ' + titleString)
    plt.savefig('trajectory0'+str(level1Index)+'_noCorrection.png', dpi=300)
    plt.savefig('trajectory0'+str(level1Index)+'_noCorrection.pdf')
    plt.close()
    
    
    # check if some frames are wrong
    if str(level1Index) in corrections.keys():
        df.loc[corrections[str(level1Index)],:] = np.nan
    
    # interpolate missing values
    nanIndices = np.where(df.nanIndices == 1)[0]
    if nanIndices or str(level1Index) in corrections.keys(): # i.e. there are some nan values to interpolate
        df = df.interpolate()
    
    # plot trajectory again
    plt.plot(df.tadpoleXcontour, df.tadpoleYcontour, color='k')
    plt.hold(True)
    plt.axes().set_aspect('equal')
    plt.ylim(maxHeight+10, -10)
    plt.xlim(-10, maxWidth+10) 
    plt.plot([0, maxWidth, maxWidth, 0, 0], [0, 0, maxHeight, maxHeight, 0], color='k')
    plt.axis('off')
    plt.title('trajectory interpolated, ' + titleString)
    plt.savefig('trajectory0'+str(level1Index)+'_interp.png', dpi=300)
    plt.close()
    
    
    # load timestamps (for all the videos that I have them... otherwise rely on framerate)
    os.chdir(timestampPath)
    try:
        ts = pd.read_csv(level1.Filename[level1Index] + '_TS.csv')
        df['time'] = np.nan
        df.loc[0, 'time'] = 0
        df.loc[1:, 'time'] = np.array(ts)
    except:
         # add time to the df
        timePerFrame = 1/level1.framerate[level1Index]
        df['time']  = np.arange(0,len(df))*timePerFrame
    os.chdir(plotsCheckFolder)
    
    # calculate distance to image edge... should only be done on the warped image!!
    # this now gives the distance to the edge in mm (as of 2016_12_04) given the tank size and the warp size
    distToEdge = []
    for myInd in range(0,len(df)):
        distToEdge.append(distanceToImageEdge((df.tadpoleXcontour[myInd],df.tadpoleYcontour[myInd]), 
                                              warpShape, level1.tankSize[level1Index]))
    
    # add distance to edge to df
    df['distToEdge'] = np.nan
    df['distToEdge'] = pd.Series(distToEdge)
    
    
    # plot a cumulative distribution of the distance to the image edge
    ecdf = sm.distributions.ECDF(df.distToEdge)
    fig, ax = plt.subplots()
    myMin = min(df.distToEdge)
    myMax = max(df.distToEdge)
    x1 = np.linspace(myMin, myMax, num=(myMax-myMin)*10)
    y = ecdf(x1)
    plt.step(x1, y, c='k')
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    ax.set_xlabel('distance to image edge (mm)')
    ax.set_ylabel('cumulative distribution')
    plt.savefig('cdfToEdge0'+str(level1Index)+'.png', dpi=300)
    plt.savefig('cdfToEdge0'+str(level1Index)+'.pdf')
    plt.close()
    
    # plot cdf for first and last quarter of video
    quarterTimeIndex = round(len(df)/4)
    ecdf1 = sm.distributions.ECDF(df.distToEdge[:quarterTimeIndex])
    ecdf2 = sm.distributions.ECDF(df.distToEdge[quarterTimeIndex:(2*quarterTimeIndex)])
    ecdf3 = sm.distributions.ECDF(df.distToEdge[(2*quarterTimeIndex):(3*quarterTimeIndex)])
    ecdf4 = sm.distributions.ECDF(df.distToEdge[(3*quarterTimeIndex):])
    y1 = ecdf1(x1)
    y2 = ecdf2(x1)
    y3 = ecdf3(x1)
    y4 = ecdf4(x1)
    fig, ax = plt.subplots()
    plt.hold(True)
    myColors = [0.35, 0.55, 0.75, 0.95]
    for myInd, myY in enumerate([y1, y2, y3, y4]):
        plt.step(x1, myY, c=cm.Greys(myColors[myInd]))
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    ax.set_xlabel('distance to image edge (mm)')
    ax.set_ylabel('cumulative distribution')
    plt.legend(['first quarter', 'second quarter', 'third quarter', 'last quarter'], frameon=False, loc='lower right')
    plt.savefig('cdfToEdgeQuartered0'+str(level1Index)+'.png', dpi=300)
    plt.savefig('cdfToEdgeQuartered0'+str(level1Index)+'.pdf')
    plt.close()
    
    
    # calculate distance covered
    dist = []
    for myInd in range(1, len(df)):
        dist.append(distCoveredMM((df.tadpoleXcontour[myInd-1], df.tadpoleYcontour[myInd-1]),
                                 (df.tadpoleXcontour[myInd], df.tadpoleYcontour[myInd]), warpShape, level1.tankSize[level1Index]))
    df['distPerFrame'] = np.nan
    df.loc[1:len(df), 'distPerFrame'] = dist
    
    # combine with time to get forward speed
    df['forwardSpeed'] = np.nan
    df.loc[1:len(df), 'forwardSpeed'] = df.loc[1:len(df), 'distPerFrame']/np.diff(df.time)
    
    """
    # plot speed separately for close to wall and further away from wall
    distThresh = 15
    try:
        fig, ax = plt.subplots()
        __ = df.forwardSpeed[df.distToEdge > distThresh].plot.hist(bins=50)
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.tick_params(axis='both', direction='out')
        ax.tick_params(top='off', right='off')
        ax.set_xlabel('speed (mm/s)')
        ax.set_ylabel('nr of frames')
        ax.set_title('speed when >' + str(distThresh) + 'mm away from wall')
        plt.savefig('speedHistAway0'+str(level1Index)+'.png', dpi=300)
        plt.close()
    except:
        print('animal not in centre!')
    
    try:
        fig, ax = plt.subplots()
        __ = df.forwardSpeed[df.distToEdge <= distThresh].plot.hist(bins=50)
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.tick_params(axis='both', direction='out')
        ax.tick_params(top='off', right='off')
        ax.set_xlabel('speed (mm/s)')
        ax.set_ylabel('nr of frames')
        ax.set_title('speed when within '+ str(distThresh) +'mm from the wall')
        plt.savefig('speedHistNear0'+str(level1Index)+'.png', dpi=300)
        plt.close()
    except:
        print('animal not near wall!')
    
    # plot speed vs. distance from wall
    fig, ax = plt.subplots()
    __ = df.plot.scatter(x='distToEdge', y='forwardSpeed', ax=ax)
    maskForRegression = df.forwardSpeed.notnull()
    z = np.polyfit(df.distToEdge[maskForRegression], df.forwardSpeed[maskForRegression], 1)
    fit_fn = np.poly1d(z)
    plt.plot(df.distToEdge[1:], df.forwardSpeed[1:], 'yo', df.distToEdge[1:], fit_fn(df.distToEdge[1:]), '--k')
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.xlabel('distance to edge (mm)')
    plt.ylabel('forward speed (mm/s)')
    # different version: get more stats
    z = linregress(df.distToEdge[maskForRegression], df.forwardSpeed[maskForRegression])
    print("r-squared:", z.rvalue**2)
    plt.text(max(dist)*0.8, max(df.forwardSpeed[1:len(df)])*0.8, 'r^2: ' + str(z.rvalue**2))
    plt.tight_layout()
    plt.savefig('speedVsDistToEdge0'+str(level1Index)+'.png', dpi=300)
    plt.close()
    
    # correct ellipse angle for jumps
    ellipseAngle = np.array(df.ellipseAngle)
    maxAngle = 145
    for myInd in range(1,len(df)):
        angleDiff = ellipseAngle[myInd] - ellipseAngle[myInd-1]
        if angleDiff > maxAngle:
            ellipseAngle[myInd:] -= 180
        elif angleDiff < -maxAngle:
            ellipseAngle[myInd:] += 180
    
    # low-pass filter ellipse angle
    # ver1: butter
    filtOrder = 3
    samplingFreq = level1.framerate[level1Index]
    bandwidth = 2
    bz, az = signal.butter(filtOrder, bandwidth/(samplingFreq/2)) # Gives you lowpass Butterworth as default
    ellipseAngleFilt = signal.filtfilt(bz, az, ellipseAngle) # Makes forward/reverse filtering (linear phase filter)
    
    fig, ax = plt.subplots()
    turningAngles = np.diff(ellipseAngleFilt)
    ax.plot(turningAngles)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.title('ellipse-based turning angles')
    plt.savefig('turningAnglesOverTime0'+str(level1Index)+'.png', dpi=300)
    plt.close()
    
    # plot histogram of turning angles
    fig, ax = plt.subplots()
    plt.hist(turningAngles, bins=100) #[np.logical_not(np.isnan(turningAngles))], bins = 100)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.xlabel('turning angle (deg)')
    plt.ylabel('number of frames')
    print('mean +- std of frame-by-frame turning angle (based on ellipse angle): \n{:.2f} +- {:.3f}!'.format(np.mean(turningAngles),np.std(turningAngles)))
    # not sure whether I would have to do a circular standard deviation and mean?
    print('circular mean +- std: {:.2f} +- {:.3f}!'.format(np.degrees(circ.mean(np.deg2rad(turningAngles))), np.degrees(circ.std(np.deg2rad(turningAngles)))))
    plt.text(-20, 200, 'circular mean +- std: {:.2f} +- {:.3f}!'.format(np.degrees(circ.mean(np.deg2rad(turningAngles))), np.degrees(circ.std(np.deg2rad(turningAngles)))))    
    plt.title('hist of ellipse-based turning angles')    
    plt.savefig('turningAnglesHist0'+str(level1Index)+'.png', dpi=300)
    plt.close()
    """
    
    # simplify trajectory using rdp#####################################
    
    # calculate epsilon appropriately to scale and resolution
    multFactor = 10/(880/190)
    pixelPerMM = max(warpShape)/tankSizeMM
    epsilon = pixelPerMM * multFactor
    print('epsilon: ', epsilon)
    saveMyFig = False
    reduced = reducePoints(df, epsilon, saveMyFig)
    
    #plot the reduced trajectory
    plt.plot(reduced.x, reduced.y, color='k')
    plt.hold(True)
    plt.axes().set_aspect('equal')
    plt.ylim(maxHeight+10, -10)
    plt.xlim(-10, maxWidth+10)
    plt.plot([0, maxWidth, maxWidth, 0, 0], [0, 0, maxHeight, maxHeight, 0], color='k')
    plt.axis('off')
    plt.title('reduced trajectory')
    plt.savefig('reducedTrajectory0'+str(level1Index)+'.png', dpi=300)
    plt.savefig('reducedTrajectory0'+str(level1Index)+'.pdf')
    plt.close()
    
    # plot the original and the reduced trajectory on top of each other
    plt.plot(df.tadpoleXcontour, df.tadpoleYcontour, color='k')
    plt.hold(True)
    plt.plot(reduced.x, reduced.y, color='b')
    plt.axes().set_aspect('equal')
    plt.ylim(maxHeight+10, -10)
    plt.xlim(-10, maxWidth+10)
    plt.plot([0, maxWidth, maxWidth, 0, 0], [0, 0, maxHeight, maxHeight, 0], color='k')
    plt.axis('off')
    plt.title('reduced and original trajectory')
    plt.savefig('reducedAndOriginalTrajectory0'+str(level1Index)+'.png', dpi=300)
    plt.savefig('reducedAndOriginalTrajectory0'+str(level1Index)+'.pdf')
    plt.close()
    
    # for the simplified trajectory calculate segment lengths
    distReduced = []
    for myInd in range(1, len(reduced)):
        distReduced.append(distCoveredMM((reduced.x[myInd-1], reduced.y[myInd-1]),
                                 (reduced.x[myInd], reduced.y[myInd]), warpShape, level1.tankSize[level1Index]))
    reduced['segDist'] = np.nan
    reduced.loc[1:len(reduced),'segDist'] = distReduced
    
    # for the simplified trajectory calculate pivot angles
    """
    headingAngles = []
    for myInd in range(1, len(reduced)):
        headingAngles.append(externalAngle((reduced.x[myInd-1], reduced.y[myInd-1]), (reduced.x[myInd], reduced.y[myInd])))
    pivotAngles = np.diff(headingAngles)
    
    # this is circular data!!!! ----------------------------------------------
    # 'compress' the pivot angles into +-180
    for myInd, myAngle in enumerate(pivotAngles):
        if myAngle < -180:
            pivotAngles[myInd] += 360
        elif myAngle > 180:
            pivotAngles[myInd] -= 360
    fig, ax = plt.subplots()
    plt.plot(pivotAngles)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.title('pivot angles')
    plt.xlabel('reduced trajectory points')
    plt.ylabel('turning angle at pivot (deg)')
    plt.savefig('pivotAngles'+str(level1Index)+'.png', dpi=300)    
    plt.close()
    
    reduced['pivotAngles'] = np.nan
    reduced.loc[1:(len(reduced)-2), 'pivotAngles'] = pivotAngles
       
    # duplicate distance to wall from df to reduced s.t. I can select the pivotAngles based on distance to wall
    reduced['distToEdge'] = np.nan
    for myInd in range(len(reduced)):
        reduced.loc[myInd, 'distToEdge'] = df.distToEdge[reduced.df_index[myInd]]
    distThresh = 15
    reducedWallIndices = np.where(reduced.distToEdge <= distThresh)
    reducedWallIndices = reducedWallIndices[0]
    reducedCentreIndices = np.where(reduced.distToEdge > distThresh)
    reducedCentreIndices = reducedCentreIndices[0]
    
    # check if the first/last index is in the list and remove if it is true, because
    # that value will be nan for the pivotAngle and the histogram function cannot deal with nans
    for myInd in [0, len(reduced)-1]:
        if myInd in reducedWallIndices: 
            reducedWallIndices = np.delete(reducedWallIndices, np.where(reducedWallIndices == myInd))
        if myInd in reducedCentreIndices: 
            reducedCentreIndices = np.delete(reducedCentreIndices, np.where(reducedCentreIndices == myInd))
    
    # separate histograms of pivot angles near/far from wall
    if reducedCentreIndices.any(): #i.e. only try and plot if not empty
        fig, ax = plt.subplots()        
        plt.hist(reduced.pivotAngles.loc[reducedCentreIndices], bins=50)
        plt.xlabel('pivot angle (deg) when away from wall')
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.tick_params(axis='both', direction='out')
        ax.tick_params(top='off', right='off')
        plt.xlim(-180,180)
        plt.savefig('pivotAnglesAway0'+str(level1Index)+'.png', dpi=300)
        plt.close()
    
    fig, ax = plt.subplots()    
    plt.hist(reduced.pivotAngles.loc[reducedWallIndices], bins=50)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.xlabel('pivot angle (deg) when near wall')
    plt.xlim(-180,180)
    plt.savefig('pivotAnglesNear0'+str(level1Index)+'.png', dpi=300)
    plt.close()
    
    # separate histograms of segment lengths near/far from wall
    commonMax = np.round(reduced.segDist.max()/10)*10
    if reducedCentreIndices.any():
        fig, ax = plt.subplots()        
        plt.hist(reduced.segDist.loc[reducedCentreIndices], bins = 50)
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.tick_params(axis='both', direction='out')
        ax.tick_params(top='off', right='off')
        plt.xlabel('segment length (mm) when away from the wall')
        plt.xlim(0, commonMax)
        plt.savefig('segmentLengthAway0'+str(level1Index)+'.png', dpi=300)
        plt.close()
    
    fig, ax = plt.subplots()    
    plt.hist(reduced.segDist.loc[reducedWallIndices], bins = 50)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.xlabel('segment length(mm) when near wall')
    plt.xlim(0, commonMax)
    plt.savefig('segmentLengthNear0'+str(level1Index)+'.png', dpi=300)
    plt.close()
    
    # scatter of pivot angles vs segment lengths, separately for near/away from the wall
    if reducedCentreIndices.any():
        fig, ax = plt.subplots()
        ax.scatter(reduced.pivotAngles.loc[reducedCentreIndices], reduced.segDist.loc[reducedCentreIndices])
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.tick_params(axis='both', direction='out')
        ax.tick_params(top='off', right='off')
        plt.xlim(-180,180)
        plt.ylim(0, commonMax)
        plt.xlabel('pivot angle (deg)')
        plt.ylabel('segment distance (mm)')
        plt.title('away from wall')
        plt.savefig('pivotVsSegmentAway0'+str(level1Index)+'.png', dpi=300)
        plt.close()
    
    fig, ax = plt.subplots()    
    plt.scatter(reduced.pivotAngles.loc[reducedWallIndices], reduced.segDist.loc[reducedWallIndices])
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    plt.xlim(-180,180)
    plt.ylim(0,commonMax)
    plt.xlabel('pivot angle (deg)')
    plt.ylabel('segment distance (mm)')
    plt.title('near wall')
    plt.savefig('pivotVsSegmentNear0'+str(level1Index)+'.png', dpi=300)
    plt.close()
    """
    
    # save level 2: 
    os.chdir(savePath)
    df.to_pickle(level1.Filename[level1Index] + '_df.pkl')
    reduced.to_pickle(level1.Filename[level1Index] + '_reduced.pkl')
    
    return df, reduced