# -*- coding: utf-8 -*-
"""
Created on Wed Dec 21 13:29:16 2016
This script contains a loop going through all the videos indicated in the level1 table,
which then loads one image from each video, asks the user to click on the 4 corners
of the tank and saves the location of those in the level1 table.

@author: haenzi
"""

import numpy as np
import cv2
import os
from os import path
import pandas as pd

# hardcoded folder locations
thigmoPath = 'D:\\SH data\\thigmo'
baseImageCheck = 'D:\\SH data\\thigmo\\analysis\\trackingCheck'
savePath = 'D:\\SH data\\thigmo\\analysis'
basePlots = 'D:\\SH data\\thigmo\\analysis\\plots'

# read level1 file
os.chdir(thigmoPath)
level1 = pd.read_csv('thigmoOverviewForPandas.csv')
level1 = level1.set_index('pythonIndex')

regularTankSizes = [7, 19]

for level1Index in np.arange(135, len(level1)):
    
    currentTankSize = level1.tankSize[level1Index]
    
    # run this if clause in case i want only to do the tanks where I added corners
    #if currentTankSize in regularTankSizes:
    #    continue
    
    # run this only if I want to do the tanks that I have not yet done
    if not np.isnan(level1.tank0x[level1Index]):
        continue
    
    runfile('D:/SH data/thigmo/python scripts/findTankCorners.py', wdir='D:/SH data/thigmo/python scripts')
    
    print('working on index ', level1Index)
    
    # get data folder
    dataPath = path.join(level1.Path[level1Index], level1.Filename[level1Index]) 
    
    # for the regular tank sizes, simply load the first image; for the other ones, load the minimum
    # intensity projection (which I did in fiji and is saved one level up from the individual images)
    if currentTankSize in regularTankSizes:
        # load one image
        if level1.file[level1Index] == 'tif':
            imList = os.listdir(dataPath)
            imList.sort()
            imPath = path.join(dataPath, imList[0])
            img = cv2.imread(imPath,0)
        elif level1.file[level1Index] == 'avi':
            dataPath += '.avi'
            cap = cv2.VideoCapture(dataPath)
            ret, frame = cap.read()
            img = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
            cap.release()
    else:
        os.chdir(dataPath)
        os.chdir('..')
        img = cv2.imread('MIN_' + level1.Filename[level1Index] + '.tif', 0)
    
    # find tank corners and warp matrix
    #if np.isnan(level1.tank0x[level1Index]):
    warpMatrix, maxWidth, maxHeight, tankCorners = findCorners(img, level1.tankSize[level1Index])
    # tank corners... do them separately
    level1.loc[level1Index, 'tank0x'] = tankCorners[0][0]
    level1.loc[level1Index, 'tank0y'] = tankCorners[0][1]
    level1.loc[level1Index, 'tank1x'] = tankCorners[1][0]
    level1.loc[level1Index, 'tank1y'] = tankCorners[1][1]
    level1.loc[level1Index, 'tank2x'] = tankCorners[2][0]
    level1.loc[level1Index, 'tank2y'] = tankCorners[2][1]
    level1.loc[level1Index, 'tank3x'] = tankCorners[3][0]
    level1.loc[level1Index, 'tank3y'] = tankCorners[3][1]
        
    #else:
     #   continue
    
    # and save
    os.chdir(thigmoPath)
    level1.to_csv('thigmoOverviewForPandas.csv')