# -*- coding: utf-8 -*-
"""
Created on Fri Feb  3 10:15:30 2017
This script compares data from animals swimming in differently sized tanks; there were
several groups of animals in sereal differently sized tanks; eventually the 9 animals
that swam both in a 7 and a 19cm tank were compared.

@author: haenzi
"""

# run prerequisites
import numpy as np
from matplotlib import pyplot as plt
import os
from os import path
import pandas as pd
import statsmodels.api as sm
import datetime as dt
from scipy import stats

runfile('D:/SH data/thigmo/python scripts/openCVtutorialUtils.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/newThigmoTrackingFunctions.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/plottingUtilities.py', wdir='D:/SH data/thigmo/python scripts')

# define function needed below for a number of animals which were tested in a number of different tanks

def severalTanks(animalIDlist, tankSizeList, readDataPath, nearWallThresh):
    
    os.chdir(readDataPath)
    
    # define arrays    
    # make arrays either just 1D for each animnal, or if one value per animal per tank
    # then make the array of animal by tank
    severalTankStageList = np.zeros((len(animalIDlist)))
    severalTankLevel1Indices = np.zeros((len(animalIDlist), len(tankSizeList)))
    severalTankcdfX = [] #np.zeros((len(animalIDlist), len(tankSizeList)))
    severalTankcdfX500 = []
    severalTankcdfY = [] #np.zeros((len(animalIDlist), len(tankSizeList)))
    severalTankcdfY500 = []    
    severalTankProportionNearWall = np.zeros((len(animalIDlist), len(tankSizeList)))
    severalTankMedianWallDist = np.zeros((len(animalIDlist), len(tankSizeList)))
    severalTankksStatistics = np.zeros((len(animalIDlist), len(tankSizeList)))
    severalTankPValues = np.zeros((len(animalIDlist), len(tankSizeList)))
    severalTankSurviveDistance = np.ones((len(animalIDlist), len(tankSizeList)))
    severalTankExclude = np.zeros((len(animalIDlist), len(tankSizeList)))
    
        
    for indAnimal, myAnimalID in enumerate(animalIDlist):
        
        thisAnimalcdfX = []
        thisAnimalcdfY = []
        thisAnimalcdfX500 = []
        thisAnimalcdfY500 = []
        
        for indTank, myTankSize in enumerate(tankSizeList):
            
            # get the level1Index for myAnimalID in myTankSize
            animalsLevel1Indices = np.where(level1.animalID == myAnimalID)[0]
            tanksLevel1Indices = np.where(level1.tankSize == myTankSize)[0]
            myIndex = [x for x in animalsLevel1Indices if x in tanksLevel1Indices]
            myIndex = myIndex[0]
            
            severalTankLevel1Indices[indAnimal,indTank] = myIndex
        
            if level1.exclude[myIndex] == 1 or level1.trackingCheck[myIndex] != 'ok':
                severalTankExclude[indAnimal, indTank] = 1
            
            try:
                df = pd.read_pickle(level1.Filename[myIndex] + '_df.pkl')
                reduced = pd.read_pickle(level1.Filename[myIndex] + '_reduced.pkl')
            except:
                print('index ', myIndex, ' has no pickle file')
                continue
            
            # exclude if the animal did not go at least once across the tank
            tankSizeMM = myTankSize*10 #need a factor of 10 because the tank size is in cm and the dist in mm
            
            if sum(reduced.segDist.dropna()) < tankSizeMM:
                print('index ', myIndex, ' not moving enough')
                severalTankSurviveDistance[indAnimal, indTank] = 0
            
            severalTankStageList[indAnimal] = level1.stage[myIndex] # hmm, overwriting this in the tank loop... never mind
            
            # calculate proportion near wall
            severalTankProportionNearWall[indAnimal, indTank] = sum(df.distToEdge < nearWallThresh)/df.shape[0]
            severalTankMedianWallDist[indAnimal, indTank] = np.median(df.distToEdge)
            
            # cdf per animal
            ecdf = sm.distributions.ECDF(df.distToEdge)
            myMin = min(df.distToEdge)
            myMax = max(df.distToEdge)
            x1 = np.linspace(myMin, myMax, num=(myMax-myMin)*10)
            x500 = np.linspace(0, tankSizeMM/2, num=500) # this will give me 500 values from zero to the centre of the tank
            
            thisAnimalcdfX.append(x1)
            thisAnimalcdfY.append(ecdf(x1))
            
            thisAnimalcdfX500.append(x500)
            thisAnimalcdfY500.append(ecdf(x500))
            
        severalTankcdfX.append(thisAnimalcdfX)
        severalTankcdfY.append(thisAnimalcdfY)
        severalTankcdfX500.append(thisAnimalcdfX500)
        severalTankcdfY500.append(thisAnimalcdfY500)
            
    return [severalTankStageList, severalTankLevel1Indices, severalTankcdfX, severalTankcdfY, severalTankcdfX500, severalTankcdfY500, severalTankProportionNearWall, severalTankMedianWallDist, severalTankksStatistics, severalTankPValues, severalTankSurviveDistance, severalTankExclude]


def getBLsFromIDs(animalIDs, level1):
    
    BLs = []    
    for myInd, myAnimal in enumerate(animalIDs):
        currentLevel1Index = np.where(level1.animalID == myAnimal)[0]        
        BLs.append(level1.length[currentLevel1Index[0]])
    
    return BLs
    
def BLsPerTankSize(BLs, tankSizes):
    BLsPerTankSize = np.ones((len(BLs), len(tankSizes)))
    for myN in np.arange(len(BLs)):
        for myM in np.arange(len(tankSizes)):
            BLsPerTankSize[myN, myM] = BLs[myN]/tankSizes[myM]
    return BLsPerTankSize
    
# hardcoded folder locations
thigmoPath = 'D:\\SH data\\thigmo'
baseImageCheck = 'D:\\SH data\\thigmo\\analysis\\trackingCheck'
readDataPath = 'D:\\SH data\\thigmo\\analysis'
baseGroupPlots = 'D:\\SH data\\thigmo\\analysis\\groupPlots'

# read level1 file
os.chdir(thigmoPath)
level1 = pd.read_csv('thigmoOverviewForPandas.csv')
level1 = level1.set_index('pythonIndex')
corrections = np.load('corrections.npy').item()

os.chdir(readDataPath)
nearWallThresh = 15 #------------------

# identify the indices of the animals where I have tanks of several sizes

# animalIDs 19, 20, 21, 22, and 23 have tank sizes 7, 10 and 19
threeTankAnimalIDs = [19, 20, 21, 22]
threeTankBLs = getBLsFromIDs(threeTankAnimalIDs, level1)
threeTankSizes = [7, 10, 19]
threeTankBLsPerTankSize = BLsPerTankSize(threeTankBLs, threeTankSizes)
threeTankData = severalTanks(threeTankAnimalIDs, threeTankSizes, readDataPath, nearWallThresh)
threeTankStageList, threeTankLevel1Indices, threeTankcdfX, threeTankcdfY, threeTankcdfX500, threeTankcdfY500, threeTankProportionNearWall, threeTankMedianWallDist, threeTankksStatistics, threeTankPValues, threeTankSurviveDistance, threeTankExclude = threeTankData

# animalIDs 24 and 25 have tank sizes 5, 6, and 7
"""
smallTankIDs = [24, 25]
smallTankBLs = getBLsFromIDs(smallTankIDs, level1)
smallTankSizes = [5, 6, 7]
smallTankBLsPerTankSize = BLsPerTankSize(smallTankBLs, smallTankSizes)
smallTankData = severalTanks(smallTankIDs, smallTankSizes, readDataPath, nearWallThresh)
smallTankStageList, smallTankLevel1Indices, smallTankcdfX, smallTankcdfY, smallTankcdfX500, smallTankcdfY500, smallTankProportionNearWall, smallTankMedianWallDist, smallTankksStatistics, smallTankPValues, smallTankSurviveDistance, smallTankExclude = smallTankData
"""

# animalIDs 26, 27, 28, 29, and 30 have tank sizes 5, 7, 12 and 19
fourTankIDs = [26, 27, 28, 29, 30]
fourTankBLs = getBLsFromIDs(fourTankIDs, level1)
fourTankSizes = [5, 7, 12, 19]
fourTankBLsPerTankSize = BLsPerTankSize(fourTankBLs, fourTankSizes)
fourTankData = severalTanks(fourTankIDs, fourTankSizes, readDataPath, nearWallThresh)
fourTankStageList, fourTankLevel1Indices, fourTankcdfX, fourTankcdfY, fourTankcdfX500, fourTankcdfY500, fourTankProportionNearWall, fourTankMedianWallDist, fourTankksStatistics, fourTankPValues, fourTankSurviveDistance, fourTankExclude = fourTankData

# make folder to save them: today's date
# get or make folder to put the plots
now = dt.datetime.now()
todaysDate = now.strftime('%Y_%m_%d')
groupPlotsFolder = path.join(baseGroupPlots, todaysDate)
if not path.exists(groupPlotsFolder):
    os.mkdir(groupPlotsFolder) 
tankSizeFolder = path.join(groupPlotsFolder, 'tankSizeComparisons')
if not path.exists(tankSizeFolder):
    os.mkdir(tankSizeFolder)
os.chdir(tankSizeFolder)

# plot proportion near wall
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(threeTankAnimalIDs)):
    ax.plot(threeTankSizes, threeTankProportionNearWall[myInd,:],'o-', c='k')
#for myInd in np.arange(len(smallTankIDs)):
 #   ax.plot(smallTankSizes, smallTankProportionNearWall[myInd,:], 'k')
for myInd in np.arange(len(fourTankIDs)):
    ax.plot(fourTankSizes, fourTankProportionNearWall[myInd,:], 'o-', c='k')
ax.set_xlabel('tank size (cm)', fontsize=14)
ax.set_ylabel('proportion within ' + str(nearWallThresh) + 'mm of wall', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.set_ylim(0,1)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('propNearWallTankSizes.png', dpi=300)
plt.savefig('propNearWallTankSizes.pdf')
plt.close()

# for statistically comparing the 7 and the 19 cm tank baased on the ks statistics and prop near wall 
tank7data = np.zeros((9,))
tank7prop = np.zeros((9,))
tank19data = np.zeros((9,))
tank19prop = np.zeros((9,))

fourTanks7Index = np.where(np.array(fourTankSizes) == 7)[0]
fourTanks19Index = np.where(np.array(fourTankSizes) == 19)[0]

threeTanks7Index = np.where(np.array(threeTankSizes) == 7)[0]
threeTanks19Index = np.where(np.array(threeTankSizes) == 19)[0]

# fill in the tank7 and tank19data
for myInd in np.arange(len(fourTankksStatistics)):
    tank7prop[myInd] = fourTankProportionNearWall[myInd][fourTanks7Index]
    tank19prop[myInd] = fourTankProportionNearWall[myInd][fourTanks19Index]
for myInd in np.arange(len(threeTankksStatistics)):
    tank7prop[myInd + len(fourTankksStatistics)] = threeTankProportionNearWall[myInd][threeTanks7Index]
    tank19prop[myInd + len(fourTankksStatistics)] = threeTankProportionNearWall[myInd][threeTanks19Index]

#plot proportion in 7 and 19 cm tank
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(tank7prop)):
    ax.plot([1,2], [tank7prop[myInd], tank19prop[myInd]], 'k')
ax.set_xlabel('tank size', fontsize=14)
ax.set_ylabel('proportion near wall', fontsize=14)
ax.set_xticks([1,2])
ax.set_xticklabels(['7 cm', '19 cm'])
ax.set_xlim(0.8, 2.2)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('propNearWall7vs19.png', dpi=300)
plt.savefig('propNearWall7vs19.pdf')

# plot proportion near wall in 7 and 19 cm tank as box-and-line plot
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(tank7prop)):
    ax.plot([1.25, 1.75], [tank7prop[myInd], tank19prop[myInd]], c='k', linewidth = 0.5)
ax.set_ylabel('proportion near wall', fontsize=14)
ax.boxplot([tank7prop, tank19prop],
           boxprops = {'color': 'black', 'linestyle': '-'}, 
            whiskerprops = {'color': 'black', 'linestyle': '-'}, 
            flierprops = {'color': 'black'})
ax.set_xticks([1,2])
ax.set_xticklabels(['7 cm','19 cm'])
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
ax.set_xlim([0.8, 2.2])
ax.set_ylim(0,1)        
plt.tight_layout()
plt.savefig('propNearWall7vs19boxAndLine.png', dpi=300)
plt.savefig('propNearWall7vs19boxAndLine.pdf')



    
# are these normally distributed?
w, p = stats.shapiro(tank7prop)
print('tank7prop: w = %g, p = %g' %(w,p)) # significant
w, p = stats.shapiro(tank19prop)
print('tank19prop: w = %g, p = %g' %(w,p)) # ns
# therefore do a paired wilcoxon
w, p = stats.wilcoxon(tank7prop, tank19prop)
print('proportion near wall in tank 7 vs. in tank 19: w = %g, p = %g' %(w,p)) # w = 1, p = 0.0108622


# read out proportion in 'nomalised' cdfs; from 15% of the length of the wall (that is basically in the middle
# between what 15mm is in 7 and 19cm, though not for area but linearly adjusted) ... actually that would be
# 0.3, since proportion of side vs. proportion of max distance to the wall is a factor of 2.. but same result
# either way
normThresh = 0.28

fourTankProp15Percent = np.zeros((fourTankProportionNearWall.shape))
for myAn in np.arange(len(fourTankcdfX500)):
    for myInd in np.arange(len(fourTankcdfX500[myAn])):
        valueNeeded = max(fourTankcdfX500[myAn][myInd])*normThresh
        indexNeeded = np.abs(fourTankcdfX500[myAn][myInd] - valueNeeded).argmin()
        if indexNeeded != 75:
            print('index is not 75')
        fourTankProp15Percent[myAn, myInd] = fourTankcdfY500[myAn][myInd][indexNeeded]

threeTankProp15Percent = np.zeros((threeTankProportionNearWall.shape))
for myAn in np.arange(len(threeTankcdfX500)):
    for myInd in np.arange(len(threeTankcdfX500[myAn])):
        valueNeeded = max(threeTankcdfX500[myAn][myInd])*normThresh
        indexNeeded = np.abs(threeTankcdfX500[myAn][myInd] - valueNeeded).argmin()
        if indexNeeded != 75:
            print('index is not 75')
        threeTankProp15Percent[myAn, myInd] = threeTankcdfY500[myAn][myInd][indexNeeded]

tank7prop15 = np.zeros((9,))
tank19prop15 = np.zeros((9,))
tank7prop15[:len(threeTankAnimalIDs)] = threeTankProp15Percent[:,threeTanks7Index].flatten()
tank7prop15[len(threeTankAnimalIDs):] = fourTankProp15Percent[:, fourTanks7Index].flatten()
tank19prop15[:len(threeTankAnimalIDs)] = threeTankProp15Percent[:,threeTanks19Index].flatten()
tank19prop15[len(threeTankAnimalIDs):] = fourTankProp15Percent[:, fourTanks19Index].flatten()


# then plot
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(tank7prop15)):
    ax.plot([1,2], [tank7prop15[myInd], tank19prop15[myInd]], 'k')
ax.set_xlabel('tank size', fontsize=14)
ax.set_ylabel('proportion near wall (15% of length)', fontsize=14)
ax.set_xticks([1,2])
ax.set_xticklabels(['7 cm', '19 cm'])
ax.set_xlim(0.8, 2.2)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('prop15NearWall7vs19.png', dpi=300)
plt.savefig('prop15NearWall7vs19.pdf')

# again as a box-and-line plot
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(tank7prop15)):
    ax.plot([1.25, 1.75], [tank7prop15[myInd], tank19prop15[myInd]], c='k', linewidth = 0.5)
ax.set_ylabel('proportion near wall', fontsize=14)
ax.boxplot([tank7prop15, tank19prop15],
           boxprops = {'color': 'black', 'linestyle': '-'}, 
            whiskerprops = {'color': 'black', 'linestyle': '-'}, 
            flierprops = {'color': 'black'})
ax.set_xticks([1,2])
ax.set_xticklabels(['7 cm','19 cm'])
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
ax.set_xlim([0.8, 2.2])
ax.set_ylim(0,1)        
plt.tight_layout()
plt.savefig('prop15NearWall7vs19boxAndLine.png', dpi=300)
plt.savefig('prop15NearWall7vs19boxAndLine.pdf')


    
# are these normally distributed?
w, p = stats.shapiro(tank7prop15)
print('tank7prop: w = %g, p = %g' %(w,p)) # significant
w, p = stats.shapiro(tank19prop15)
print('tank19prop: w = %g, p = %g' %(w,p)) # ns
# therefore do a paired wilcoxon
w, p = stats.wilcoxon(tank7prop15, tank19prop15)
print('proportion near wall in tank 7 vs. in tank 19: w = %g, p = %g' %(w,p)) # w = 0, p = 0.0076857
   
   
# plot cdfs on same scale
fig, ax = plt.subplots()
ax.hold(True)
myColors = ['k','r']
fourTank7index = np.where(np.array(fourTankSizes) == 7)[0]
fourTank19index = np.where(np.array(fourTankSizes) == 19)[0]
for myAn in np.arange(len(fourTankcdfX500)):
    ax.plot(np.linspace(0,1,500), fourTankcdfY500[myAn][fourTank7index], c='k')
    ax.plot(np.linspace(0,1,500), fourTankcdfY500[myAn][fourTank19index], c='b')
threeTank7index = np.where(np.array(threeTankSizes) == 7)[0]
threeTank19index = np.where(np.array(threeTankSizes) == 19)[0]
for myAn in np.arange(len(threeTankcdfX500)):
    ax.plot(np.linspace(0,1,500), threeTankcdfY500[myAn][threeTank7index], c='k')
    ax.plot(np.linspace(0,1,500), threeTankcdfY500[myAn][threeTank19index], c='b')
ax.plot([normThresh, normThresh], [0,1], '--', c='g')
ax.set_xlabel('distance from the nearest wall', fontsize=14)
ax.set_ylabel('cumulative frequency', fontsize=14)
ax.legend(['7 cm tank', '19 cm tank'], loc=4, frameon=False)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('cdfNorm7vs19.png', dpi=300)
plt.savefig('cdfNorm7vs19.pdf')


# plot cdfs on original scale
fig, ax = plt.subplots()
ax.hold(True)
fourTank7index = np.where(np.array(fourTankSizes) == 7)[0]
fourTank19index = np.where(np.array(fourTankSizes) == 19)[0]
for myAn in np.arange(len(fourTankcdfX500)):
    ax.plot(fourTankcdfX500[myAn][fourTank7index], fourTankcdfY500[myAn][fourTank7index], c='k')
    ax.plot(fourTankcdfX500[myAn][fourTank19index], fourTankcdfY500[myAn][fourTank19index], c='b')
threeTank7index = np.where(np.array(threeTankSizes) == 7)[0]
threeTank19index = np.where(np.array(threeTankSizes) == 19)[0]
for myAn in np.arange(len(threeTankcdfX500)):
    ax.plot(threeTankcdfX500[myAn][threeTank7index], threeTankcdfY500[myAn][threeTank7index], c='k')
    ax.plot(threeTankcdfX500[myAn][threeTank19index], threeTankcdfY500[myAn][threeTank19index], c='b')
ax.plot([nearWallThresh, nearWallThresh], [0,1], '--', c='g')
ax.set_xlabel('distance from the nearest wall (mm)', fontsize=14)
ax.set_ylabel('cumulative frequency', fontsize=14)
ax.legend(['7 cm tank', '19 cm tank'], loc=4, frameon=False)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('cdf7vs19.png', dpi=300)
plt.savefig('cdf7vs19.pdf')
    

# plot cdfs on same scale plus expectation
myMin = 0
nSamples = 9000
myMax = 1
np.random.seed(10)
myX = np.random.uniform(myMin, myMax, nSamples)
myY = np.random.uniform(myMin, myMax, nSamples)
allDistances = np.array([myX, myY, myMax - myX, myMax - myY])
distancesToWall = np.min(allDistances, axis=0)
ecdf = sm.distributions.ECDF(distancesToWall)
xExpect = np.linspace(myMin, myMax, num=500)

fig, ax = plt.subplots()
ax.hold(True)
ax.plot(xExpect, ecdf(xExpect), 'r')
fourTank7index = np.where(np.array(fourTankSizes) == 7)[0]
fourTank19index = np.where(np.array(fourTankSizes) == 19)[0]
for myAn in np.arange(len(fourTankcdfX500)):
    ax.plot(np.linspace(0,1,500), fourTankcdfY500[myAn][fourTank7index], c='k')
    ax.plot(np.linspace(0,1,500), fourTankcdfY500[myAn][fourTank19index], c='b')
threeTank7index = np.where(np.array(threeTankSizes) == 7)[0]
threeTank19index = np.where(np.array(threeTankSizes) == 19)[0]
for myAn in np.arange(len(threeTankcdfX500)):
    ax.plot(np.linspace(0,1,500), threeTankcdfY500[myAn][threeTank7index], c='k')
    ax.plot(np.linspace(0,1,500), threeTankcdfY500[myAn][threeTank19index], c='b')
ax.set_xlabel('distance from the nearest wall', fontsize=14)
ax.set_ylabel('cumulative frequency', fontsize=14)
ax.legend(['expectation', '7 cm tank', '19 cm tank'], loc=4, frameon=False)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.tight_layout()
plt.savefig('cdfNorm7vs19plusExpectation.png', dpi=300)
plt.savefig('cdfNorm7vs19plusExpectation.pdf')



# sanity check: plot ks stats of 7 vs. 19 the same way
fig, ax = plt.subplots()
ax.hold(True)
for myAn in np.arange(len(fourTankksStatistics)):
    ax.plot([1,2], [fourTankksStatistics[myAn, fourTank7index], fourTankksStatistics[myAn, fourTank19index]], c='k')
for myAn in np.arange(len(threeTankksStatistics)):
    ax.plot([1,2], [threeTankksStatistics[myAn, threeTank7index], threeTankksStatistics[myAn, threeTank19index]], c='k')


# plotting examples ----------------------------------------------------------------
# plot cdfs (on same scale!) and trajectories for example
# e.g. animalID 20, in 7cm tank (level1Index 32) and in 19 cm tank (level1Index 33)


# run function to plot

def plotTankSizeExamples(df, level1Index, level1):

    tankCorners = [[0, 0], [0, 0], [0, 0], [0, 0]]
    tankCorners[0][0] = level1.tank0x[level1Index]
    tankCorners[0][1] = level1.tank0y[level1Index]
    tankCorners[1][0] = level1.tank1x[level1Index]
    tankCorners[1][1] = level1.tank1y[level1Index]
    tankCorners[2][0] = level1.tank2x[level1Index]
    tankCorners[2][1] = level1.tank2y[level1Index]
    tankCorners[3][0] = level1.tank3x[level1Index]
    tankCorners[3][1] = level1.tank3y[level1Index]
    warpMatrix, maxWidth, maxHeight = reconstructWarp(tankCorners)

    plt.plot(df.tadpoleXcontour, df.tadpoleYcontour, color='k')
    plt.hold(True)
    plt.axes().set_aspect('equal')
    plt.ylim(maxHeight+10, -10)
    plt.xlim(-10, maxWidth+10) 
    plt.plot([0, maxWidth, maxWidth, 0, 0], [0, 0, maxHeight, maxHeight, 0], color='k')
    plt.axis('off')
    plt.title('trajectory interpolated')
    plt.savefig('trajectory0'+str(level1Index)+'_interp.png', dpi=300)
    plt.savefig('trajectory0'+str(level1Index)+'_interp.pdf')
    plt.close()
    
    # plot a cumulative distribution of the distance to the image edge
    ecdf = sm.distributions.ECDF(df.distToEdge)
    fig, ax = plt.subplots()
    myMin = min(df.distToEdge)
    myMax = max(df.distToEdge)
    x1 = np.linspace(myMin, myMax, num=(myMax-myMin)*10)
    y = ecdf(x1)
    plt.step(x1, y, c='k')
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    ax.set_xlabel('distance to image edge (mm)')
    ax.set_ylabel('cumulative distribution')
    ax.set_xlim(0,100)
    plt.savefig('cdfToEdge0'+str(level1Index)+'.png', dpi=300)
    plt.savefig('cdfToEdge0'+str(level1Index)+'.pdf')
    plt.close()
    
def plot2cdfsSameScale(df1, df2, tankSize1, tankSize2, plotUniform):
    ecdf1 = sm.distributions.ECDF(df1.distToEdge)
    ecdf2 = sm.distributions.ECDF(df2.distToEdge)
    fig, ax = plt.subplots()
    myMin1 = min(df1.distToEdge)
    myMax1 = max(df1.distToEdge)
    x1 = np.linspace(myMin1, myMax1, num=(myMax1-myMin1)*10)
    y1 = ecdf1(x1)
    plt.step(x1, y1, c='g')
    plt.hold(True)
    ecdf2 = sm.distributions.ECDF(df2.distToEdge)
    myMin2 = min(df2.distToEdge)
    myMax2 = max(df2.distToEdge)
    x2 = np.linspace(myMin2, myMax2, num=(myMax2-myMin2)*10)
    y2 = ecdf2(x2)
    plt.step(x2, y2, c='b')
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.tick_params(axis='both', direction='out')
    ax.tick_params(top='off', right='off')
    ax.set_xlabel('distance to image edge (mm)')
    ax.set_ylabel('cumulative distribution')
    ax.set_xlim(0,100)
    
    
    if plotUniform == True:
        # then, set the seed for the numpy random number generator
        tankSizes = [tankSize1, tankSize2]
        for myN in (0,1):
            myMax = tankSizes[myN]*10
            np.random.seed(10)
            myX = np.random.uniform(0, myMax, 9000)
            myY = np.random.uniform(0, myMax, 9000)
        
            # calculate wall distances and plot as hist
            allDistances = np.array([myX, myY, myMax - myX, myMax - myY])
            distancesToWall = np.min(allDistances, axis=0)
            
            ecdf = sm.distributions.ECDF(distancesToWall)
            x1 = np.linspace(0, myMax, num=(myMax)*10)
            y = ecdf(x1)
            plt.step(x1, y, c='k')

    plt.savefig('cdfToEdgeCombo.png', dpi=300)
    plt.savefig('cdfToEdgeCombo.pdf')
    plt.close()

#load example data
os.chdir(readDataPath)
ex7level1Index = 32
ex19level1Index = 33
df7 = pd.read_pickle(level1.Filename[ex7level1Index] + '_df.pkl')
df19 = pd.read_pickle(level1.Filename[ex19level1Index] + '_df.pkl')
os.chdir(tankSizeFolder)
plotTankSizeExamples(df7, ex7level1Index, level1)
plotTankSizeExamples(df19, ex19level1Index, level1)

plot2cdfsSameScale(df7, df19, 7, 19, True)