# -*- coding: utf-8 -*-
"""
Created on Thu Feb  2 09:32:55 2017
This script compares tadpoles with tentacles in the standard condition with tadpoles
of the same developmental stages without tentacles also swimming in the standard condition.

@author: haenzi
"""


# run prerequisites
import numpy as np
from matplotlib import pyplot as plt
import os
from os import path
import pandas as pd
import statsmodels.api as sm
import datetime as dt
from scipy import stats

runfile('D:/SH data/thigmo/python scripts/openCVtutorialUtils.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/newThigmoTrackingFunctions.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/plottingUtilities.py', wdir='D:/SH data/thigmo/python scripts')

# hardcoded folder locations
thigmoPath = 'D:\\SH data\\thigmo'
baseImageCheck = 'D:\\SH data\\thigmo\\analysis\\trackingCheck'
readDataPath = 'D:\\SH data\\thigmo\\analysis'
baseGroupPlots = 'D:\\SH data\\thigmo\\analysis\\groupPlots'

# read level1 file
os.chdir(thigmoPath)
level1 = pd.read_csv('thigmoOverviewForPandas.csv')
level1 = level1.set_index('pythonIndex')
corrections = np.load('corrections.npy').item()

os.chdir(readDataPath)

primaryIndices = np.where(level1.primary == 1)[0]

# find indices of animals with tents
tentIndices = np.where(level1.tentShort > 3)[0]

# need to check whether in primary indices, otherwise include the same animal in the dark etc.
tentIndicesKeep = [x for x in tentIndices if x in primaryIndices]
tentIndices = tentIndicesKeep

stageTentAnimals = level1.stage[tentIndices]
uniqueStageTentAnimals = np.unique(stageTentAnimals)
minStageTent = min(stageTentAnimals)
maxStageTent = max(stageTentAnimals)

# find animals of those stages without tentacles (i.e. smaller than 1mm)
rightStage = [myInd for myInd in np.arange(len(level1)) if level1.stage[myInd] in np.arange(minStageTent, maxStageTent+1)]
rightTent = np.where(level1.tentLong < 1)[0]
noTentIndices = [x for x in rightStage if x in rightTent] # intersection of the two conditions above

noTentIndicesKeep = [x for x in noTentIndices if x in primaryIndices]
noTentIndices = noTentIndicesKeep

# loop through those selected data and calculate the desired values

# since these values for tents and no tents come from different animals, I will put them in different lists

#initialise
stageListTent = []
level1IndicesTent = []
animalIDsTent = []
cdfXTent = []
cdfYTent = []
proportionNearWallTent = []

stageListNoTent = []
level1IndicesNoTent = []
animalIDsNoTent = []
cdfXNoTent = []
cdfYNoTent = []
proportionNearWallNoTent = []

#set parameters outside the loop
nearWallThresh = 15
os.chdir(readDataPath)

for myIndex in (list(tentIndices) + noTentIndices):
    
    if level1.exclude[myIndex] == 1 or level1.trackingCheck[myIndex] != 'ok':
        continue
    
    try:
        df = pd.read_pickle(level1.Filename[myIndex] + '_df.pkl')
        reduced = pd.read_pickle(level1.Filename[myIndex] + '_reduced.pkl')
    except:
        print('index ', myIndex, ' has no pickle file')
        continue
    
    # exclude if the animal did not go at least once across the tank
    tankSizeMM = level1.tankSize[myIndex]*10 #need a factor of 10 because the tank size is in cm and the dist in mm
    
    if sum(reduced.segDist.dropna()) < tankSizeMM:
        print('index ', myIndex, ' not moving enough')
        continue
    
    if myIndex in tentIndices:
        
        stageListTent.append(level1.stage[myIndex])
        level1IndicesTent.append(myIndex)
        animalIDsTent.append(level1.animalID[myIndex])
        
        # calculate proportion near wall
        proportionNearWallTent.append(sum(df.distToEdge < nearWallThresh)/df.shape[0])
        
        # cdf per animal
        ecdf = sm.distributions.ECDF(df.distToEdge)
        myMin = min(df.distToEdge)
        myMax = max(df.distToEdge)
        x1 = np.linspace(myMin, myMax, num=(myMax-myMin)*10)
        cdfXTent.append(x1)
        cdfYTent.append(ecdf(x1))
        
    elif myIndex in noTentIndices:
        
        stageListNoTent.append(level1.stage[myIndex])
        level1IndicesNoTent.append(myIndex)
        animalIDsNoTent.append(level1.animalID[myIndex])
        
        # calculate proportion near wall
        proportionNearWallNoTent.append(sum(df.distToEdge < nearWallThresh)/df.shape[0])
        
        # cdf per animal
        ecdf = sm.distributions.ECDF(df.distToEdge)
        myMin = min(df.distToEdge)
        myMax = max(df.distToEdge)
        x1 = np.linspace(myMin, myMax, num=(myMax-myMin)*10)
        cdfXNoTent.append(x1)
        cdfYNoTent.append(ecdf(x1))
        
        
            
# make folder to save them: today's date
# get or make folder to put the plots
now = dt.datetime.now()
todaysDate = now.strftime('%Y_%m_%d')
groupPlotsFolder = path.join(baseGroupPlots, todaysDate)
if not path.exists(groupPlotsFolder):
    os.mkdir(groupPlotsFolder) 
tentFolder = path.join(groupPlotsFolder, 'tentComparisons')
if not path.exists(tentFolder):
    os.mkdir(tentFolder)
os.chdir(tentFolder)

# cdf
fig, ax = plt.subplots()
ax.hold(True)
for myInd in np.arange(len(cdfXTent)):
    ax.plot(cdfXTent[myInd], cdfYTent[myInd], c='k')
for myInd in np.arange(len(cdfXNoTent)):
    ax.plot(cdfXNoTent[myInd], cdfYNoTent[myInd], c=[0.7, 0.7, 0.7])
ax.set_xlabel('distance from wall (mm)', fontsize=14)
ax.set_ylabel('cdf', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
plt.legend(['bla', 'bla'], loc='lower right', frameon=False)
plt.tight_layout()
plt.savefig('cdf_tent.png', dpi=300)
plt.savefig('cdf_tent.pdf')
plt.close()

# make boxplots of the thigmo measures
# plot proportion near wall
fig, ax = plt.subplots()
ax.boxplot([proportionNearWallTent, proportionNearWallNoTent], 
           boxprops = {'color': 'black', 'linestyle': '-'}, 
            whiskerprops = {'color': 'black', 'linestyle': '-'}, 
            flierprops = {'color': 'black'})
ax.set_xticks([1,2])
ax.set_xticklabels(['tentacles', 'no tentacles'])
ax.set_ylabel('proportion near wall', fontsize=14)
for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    label.set_fontsize(14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.tick_params(axis='both', direction='out')
ax.tick_params(top='off', right='off')
ax.set_xlim([0.6, 2.4])
ax.set_ylim(0,1)
plt.tight_layout()
plt.savefig('proportionNearWallTent.png', dpi=300)
plt.savefig('proportionNearWallTent.pdf')
 
# statistics ---------------------------------------------------
 
# proportion near wall
w, p = stats.shapiro(proportionNearWallTent)
print('proportion near wall, tent: w = %g, p = %g' %(w,p)) #  not significant (p = 0.17)
w, p = stats.shapiro(proportionNearWallNoTent)
print('proportion near wall, no tent: w = %g, p = %g' %(w,p)) #also just about not significant (p = 0.067)
# therefore do a mann whitney u test:
t, p = stats.mannwhitneyu(proportionNearWallTent, proportionNearWallNoTent)
print('proportion near wall, tent vs. no tent: t = %g, p = %g' %(t,p)) # ns t = -1.56268, p = 0.132398
