# -*- coding: utf-8 -*-
"""
Created on Wed Dec 21 13:29:16 2016
This script contains a loop to try and track the animals in all videos indicated in
the level1 table. The strategies for tracking have been very much empirically adjusted
to what worked.

@author: haenzi
"""

import numpy as np
import cv2
from matplotlib import pyplot as plt
import os
from os import path
import pandas as pd
import time
import statsmodels.api as sm
import datetime as dt
#from mpl_toolkits.mplot3d import Axes3D
from scipy import signal
from scipy.stats import linregress
from rdp import rdp
import pycircstat as circ

runfile('D:/SH data/thigmo/python scripts/openCVtutorialUtils.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/newThigmoTrackingFunctions.py', wdir='D:/SH data/thigmo/python scripts')
runfile('D:/SH data/thigmo/python scripts/aviTrackingUtilities.py', wdir='D:/SH data/thigmo/python scripts')

# hardcoded folder locations
thigmoPath = 'D:\\SH data\\thigmo'
baseImageCheck = 'Y:\\Backup stuff\\Data\\SH data\\thigmoTrackingCheck' #'D:\\SH data\\thigmo\\analysis\\trackingCheck'
savePathOriginals = 'D:\\SH data\\thigmo\\analysis\\originalDFs'
basePlots = 'D:\\SH data\\thigmo\\analysis\\plots'

# read level1 file
os.chdir(thigmoPath)
level1 = pd.read_csv('thigmoOverviewForPandas.csv')
level1 = level1.set_index('pythonIndex')

for level1Index in np.arange(0, len(level1)):
    
    # uncomment this code for only tracking animals in standard condition
    #if not level1.primary[level1Index] == 1:
    #    continue
    
    if level1.trackingCheck[level1Index] == 'ok':
        continue
    
    runfile('D:/SH data/thigmo/python scripts/findTankCorners.py', wdir='D:/SH data/thigmo/python scripts')
    
    print('working on index ', level1Index)
    print('filename: ', level1.Filename[level1Index])
    
    # get data folder
    dataPath = path.join(level1.Path[level1Index], level1.Filename[level1Index])
    
    # get or make folder to put the images to check the tracking
    imageCheckFolder = path.join(baseImageCheck, level1.Filename[level1Index] + '_check')
    if not path.exists(imageCheckFolder):
        os.mkdir(imageCheckFolder) 
    
    # load one image
    if level1.file[level1Index] == 'tif':
        imList = os.listdir(dataPath)
        imList.sort()
        imPath = path.join(dataPath, imList[0])
        img = cv2.imread(imPath,0)
    elif level1.file[level1Index] == 'avi':
        dataPath += '.avi'
        cap = cv2.VideoCapture(dataPath)
        ret, frame = cap.read()
        img = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        cap.release()
    
    # find tank corners and warp matrix
    if np.isnan(level1.tank0x[level1Index]):
        warpMatrix, maxWidth, maxHeight, tankCorners = findCorners(img, level1.tankSize[level1Index])
        # tank corners... do them separately
        level1.loc[level1Index, 'tank0x'] = tankCorners[0][0]
        level1.loc[level1Index, 'tank0y'] = tankCorners[0][1]
        level1.loc[level1Index, 'tank1x'] = tankCorners[1][0]
        level1.loc[level1Index, 'tank1y'] = tankCorners[1][1]
        level1.loc[level1Index, 'tank2x'] = tankCorners[2][0]
        level1.loc[level1Index, 'tank2y'] = tankCorners[2][1]
        level1.loc[level1Index, 'tank3x'] = tankCorners[3][0]
        level1.loc[level1Index, 'tank3y'] = tankCorners[3][1]
        
    else:
        # reconstruct warpMatrix, maxWidth and maxHeight from tankCorners
        tankCorners = [[0, 0], [0, 0], [0, 0], [0, 0]]
        tankCorners[0][0] = level1.tank0x[level1Index]
        tankCorners[0][1] = level1.tank0y[level1Index]
        tankCorners[1][0] = level1.tank1x[level1Index]
        tankCorners[1][1] = level1.tank1y[level1Index]
        tankCorners[2][0] = level1.tank2x[level1Index]
        tankCorners[2][1] = level1.tank2y[level1Index]
        tankCorners[3][0] = level1.tank3x[level1Index]
        tankCorners[3][1] = level1.tank3y[level1Index]
        warpMatrix, maxWidth, maxHeight = reconstructWarp(tankCorners)
    
    stage = level1.stage[level1Index]
    tankSize = level1.tankSize[level1Index]
    
    thresh = 95
    listLowThresh = [19, 27]
    listVeryLowThreshold = [28, 37, 38]
    listNormalThreshold = [89, 92, 93]
    list99threshold = []
    list101threshold = []
    list102threshold = []
    list103threshold = [152, 149]
    list104threshold = []
    list105threshold = [114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126,
       127, 128, 129, 130, 131, 132, 133]
    list106threshold = []
    list108threshold = [104, 105, 106, 107, 108, 110, 111, 112, 113]
    list109threshold = [100]
    list110threshold = [103, 138, 139, 144, 154]
    list111threshold = [98]
    list112threshold = [102, 109, 141, 145]
    list113threshold = [134, 135]
    list114threshold = [140, 142]
    list115threshold = [136]
    #listNoSimpleThreshold = [94, 95, 96, 97, 99, 100, 103]
    if level1Index in listLowThresh:
        thresh = 65
    elif level1Index in listVeryLowThreshold:
        thresh = 50
    #elif (level1.stage[level1Index] == 50) and (level1.light[level1Index] == 'f'):
     #   thresh = 75
    elif (stage < 50) and (level1.light[level1Index] == 'f') and (tankSize > 15):
        thresh = 160
    elif level1Index in list99threshold:
        thresh = 99
    elif level1Index in list101threshold:
        thresh = 101
    elif level1Index in list102threshold:
        thresh = 102
    elif level1Index in list103threshold:
        thresh = 103
    elif level1Index in list104threshold:
        thresh = 104
    elif level1Index in list105threshold:
        thresh = 105
    elif level1Index in list106threshold:
        thresh = 106
    elif level1Index in list108threshold:
        thresh = 108
    elif level1Index in list109threshold:
        thresh = 109
    elif level1Index in list110threshold:        
        thresh = 110
    elif level1Index in list111threshold:
        thresh = 111
    elif level1Index in list112threshold:
        thresh = 112
    elif level1Index in list114threshold:
        thresh = 114
    elif level1Index in list113threshold:
        thresh = 113
    elif level1Index in list115threshold:
        thresh = 115
    elif level1.light[level1Index] == 'IR':
        thresh = 108
    elif level1.file[level1Index] == 'avi':
        thresh = 114
    elif level1.light[level1Index] == 'k' and level1Index not in listNormalThreshold:
        thresh = 65
        
    
    # run function to analyse and check
    t1 = time.time()
    #df = trackTadpoleWarp(dataPath, imageCheckFolder, warpMatrix, maxWidth, maxHeight, thresh)
    #df, warpShape = trackTadpoleBGwarp(dataPath, imageCheckFolder, bg, thresh, warpMatrix, maxWidth, maxHeight)
    listNoBGsubtraction = [19, 27, 28, 37, 38, 40, 54, 55, 63, 89, 90, 91, 92, 93]
    if (level1Index in listNoBGsubtraction):
        df, warpShape = trackTadpoleWarpPadded(dataPath, imageCheckFolder, thresh, warpMatrix, maxWidth, 
                                             maxHeight)
    elif level1.light[level1Index] == 'IR':
        df, warpShape = trackAviWarpPaddedIR(dataPath, imageCheckFolder, thresh, warpMatrix, maxWidth, maxHeight, level1Index)
    
    elif level1.file[level1Index] == 'avi':
        
        # strategy 2: no bg subtraction
        #thresh = 114
        df, warpShape = trackAviWarpPadded(dataPath, imageCheckFolder, thresh, warpMatrix, maxWidth, maxHeight)
    else:
        # find background
        spreadIndices = True
        if level1Index == 43 or level1Index == 40 or level1Index == 39:
            spreadIndices = False
        bg = findBGauto(dataPath, spreadIndices) 
        
        if level1.light[level1Index] != 'f':
            param1 = 65
            param2 = 25
            bg = makeBWthreshold(bg, param1, param2)
        
        # adjust kernel size: larger if animal is large/old
        smallerKernelList = [59]
        if tankSize < 19 and level1Index not in smallerKernelList:
            kernelSize = 8    
        elif stage > 56:
            kernelSize = 6
        elif stage > 53 or level1Index in smallerKernelList:
            kernelSize = 5
        elif stage > 50:
            kernelSize = 4
        elif stage == 50:
            kernelSize = 3
        else:
            kernelSize = 2 
            
        if level1.light[level1Index] != 'f':
            df, warpShape = trackTadpoleBWBGwarpPadded(dataPath, imageCheckFolder, bg, warpMatrix, maxWidth, 
                                             maxHeight, stage, tankSize)
        else:
            df, warpShape = trackTadpoleBGwarpPadded(dataPath, imageCheckFolder, bg, thresh, warpMatrix, maxWidth, 
                                             maxHeight, stage, tankSize, kernelSize)
        
    t2 = time.time()
    elapsed = t2-t1
    print(elapsed)
    
    # find indices where the tracking did not work properly: nan (did not find anything)
    # or ellipse and contour coordinates too far apart
    nanIndices, problemIndices = nanProblemIndices(df)
    df['nanIndices'] = 0
    df['problemIndices'] = 0
    df.loc[nanIndices, 'nanIndices'] = 1
    df.loc[problemIndices, 'problemIndices'] = 1
    
    # save level 2: 
    os.chdir(savePathOriginals)
    df.to_pickle(level1.Filename[level1Index] + '_original_df.pkl')
    
    # load: df = pd.read_pickle(level1.Filename[level1Index] + '_df.pkl')
    # load reduced: reduced = pd.read_pickle(level1.Filename[level1Index] + '_reduced.pkl')
    
    # save level 1
    # add stuff to the level 1 dataframe and save
    # date of last time analysis was run
    now = dt.datetime.now()
    todaysDate = now.strftime('%Y_%m_%d')
    level1.loc[level1Index, 'dateAnalysisLastRun'] = todaysDate
    
    # threshold used
    level1.loc[level1Index, 'threshold'] = thresh
    
    # and save
    os.chdir(thigmoPath)
    level1.to_csv('thigmoOverviewForPandas.csv')
    
    print('done with index ', level1Index)
    
    #del(df, reduced)


# this is for adding stuff to the level1 table that was not there initially from excel
# but only need to run this once
#level1.dateAnalysisLastRun = level1.dateAnalysisLastRun.astype(object)